"use strict";
(self["webpackChunk_datalayer_jupyter_editor"] = self["webpackChunk_datalayer_jupyter_editor"] || []).push([["vendors-node_modules_jupyterlite_pyodide-kernel_lib_index_js"],{

/***/ "../../../node_modules/@jupyterlite/kernel/lib/kernel.js":
/*!***************************************************************!*\
  !*** ../../../node_modules/@jupyterlite/kernel/lib/kernel.js ***!
  \***************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "BaseKernel": () => (/* binding */ BaseKernel)
/* harmony export */ });
/* harmony import */ var _jupyterlab_services__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @jupyterlab/services */ "webpack/sharing/consume/default/@jupyterlab/services");
/* harmony import */ var _jupyterlab_services__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(_jupyterlab_services__WEBPACK_IMPORTED_MODULE_0__);
/* harmony import */ var _lumino_signaling__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @lumino/signaling */ "webpack/sharing/consume/default/@lumino/signaling");
/* harmony import */ var _lumino_signaling__WEBPACK_IMPORTED_MODULE_1___default = /*#__PURE__*/__webpack_require__.n(_lumino_signaling__WEBPACK_IMPORTED_MODULE_1__);


/**
 * A base kernel class handling basic kernel messaging.
 */
class BaseKernel {
    /**
     * Construct a new BaseKernel.
     *
     * @param options The instantiation options for a BaseKernel.
     */
    constructor(options) {
        this._history = [];
        this._executionCount = 0;
        this._isDisposed = false;
        this._disposed = new _lumino_signaling__WEBPACK_IMPORTED_MODULE_1__.Signal(this);
        this._parentHeader = undefined;
        this._parent = undefined;
        const { id, name, location, sendMessage } = options;
        this._id = id;
        this._name = name;
        this._location = location;
        this._sendMessage = sendMessage;
    }
    /**
     * A promise that is fulfilled when the kernel is ready.
     */
    get ready() {
        return Promise.resolve();
    }
    /**
     * Return whether the kernel is disposed.
     */
    get isDisposed() {
        return this._isDisposed;
    }
    /**
     * A signal emitted when the kernel is disposed.
     */
    get disposed() {
        return this._disposed;
    }
    /**
     * Get the kernel id
     */
    get id() {
        return this._id;
    }
    /**
     * Get the name of the kernel
     */
    get name() {
        return this._name;
    }
    /**
     * The location in the virtual filesystem from which the kernel was started.
     */
    get location() {
        return this._location;
    }
    /**
     * The current execution count
     */
    get executionCount() {
        return this._executionCount;
    }
    /**
     * Get the last parent header
     */
    get parentHeader() {
        return this._parentHeader;
    }
    /**
     * Get the last parent message (mimic ipykernel's get_parent)
     */
    get parent() {
        return this._parent;
    }
    /**
     * Dispose the kernel.
     */
    dispose() {
        if (this.isDisposed) {
            return;
        }
        this._isDisposed = true;
        this._disposed.emit(void 0);
    }
    /**
     * Handle an incoming message from the client.
     *
     * @param msg The message to handle
     */
    async handleMessage(msg) {
        this._busy(msg);
        this._parent = msg;
        const msgType = msg.header.msg_type;
        switch (msgType) {
            case 'kernel_info_request':
                await this._kernelInfo(msg);
                break;
            case 'execute_request':
                await this._execute(msg);
                break;
            case 'input_reply':
                this.inputReply(msg.content);
                break;
            case 'inspect_request':
                await this._inspect(msg);
                break;
            case 'is_complete_request':
                await this._isCompleteRequest(msg);
                break;
            case 'complete_request':
                await this._complete(msg);
                break;
            case 'history_request':
                await this._historyRequest(msg);
                break;
            case 'comm_open':
                await this.commOpen(msg);
                break;
            case 'comm_msg':
                await this.commMsg(msg);
                break;
            case 'comm_close':
                await this.commClose(msg);
                break;
            default:
                break;
        }
        this._idle(msg);
    }
    /**
     * Stream an event from the kernel
     *
     * @param parentHeader The parent header.
     * @param content The stream content.
     */
    stream(content, parentHeader = undefined) {
        var _a;
        const parentHeaderValue = typeof parentHeader !== 'undefined' ? parentHeader : this._parentHeader;
        const message = _jupyterlab_services__WEBPACK_IMPORTED_MODULE_0__.KernelMessage.createMessage({
            channel: 'iopub',
            msgType: 'stream',
            // TODO: better handle this
            session: (_a = parentHeaderValue === null || parentHeaderValue === void 0 ? void 0 : parentHeaderValue.session) !== null && _a !== void 0 ? _a : '',
            parentHeader: parentHeaderValue,
            content,
        });
        this._sendMessage(message);
    }
    /**
     * Send a `display_data` message to the client.
     *
     * @param parentHeader The parent header.
     * @param content The display_data content.
     */
    displayData(content, parentHeader = undefined) {
        var _a, _b;
        // Make sure metadata is always set
        const parentHeaderValue = typeof parentHeader !== 'undefined' ? parentHeader : this._parentHeader;
        content.metadata = (_a = content.metadata) !== null && _a !== void 0 ? _a : {};
        const message = _jupyterlab_services__WEBPACK_IMPORTED_MODULE_0__.KernelMessage.createMessage({
            channel: 'iopub',
            msgType: 'display_data',
            // TODO: better handle this
            session: (_b = parentHeaderValue === null || parentHeaderValue === void 0 ? void 0 : parentHeaderValue.session) !== null && _b !== void 0 ? _b : '',
            parentHeader: parentHeaderValue,
            content,
        });
        this._sendMessage(message);
    }
    /**
     * Send a `input_request` message to the client.
     *
     * @param parentHeader The parent header.
     * @param content The input_request content.
     */
    inputRequest(content, parentHeader = undefined) {
        var _a;
        const parentHeaderValue = typeof parentHeader !== 'undefined' ? parentHeader : this._parentHeader;
        const message = _jupyterlab_services__WEBPACK_IMPORTED_MODULE_0__.KernelMessage.createMessage({
            channel: 'stdin',
            msgType: 'input_request',
            // TODO: better handle this
            session: (_a = parentHeaderValue === null || parentHeaderValue === void 0 ? void 0 : parentHeaderValue.session) !== null && _a !== void 0 ? _a : '',
            parentHeader: parentHeaderValue,
            content,
        });
        this._sendMessage(message);
    }
    /**
     * Send an `execute_result` message.
     *
     * @param parentHeader The parent header.
     * @param content The execute result content.
     */
    publishExecuteResult(content, parentHeader = undefined) {
        var _a;
        const parentHeaderValue = typeof parentHeader !== 'undefined' ? parentHeader : this._parentHeader;
        const message = _jupyterlab_services__WEBPACK_IMPORTED_MODULE_0__.KernelMessage.createMessage({
            channel: 'iopub',
            msgType: 'execute_result',
            // TODO: better handle this
            session: (_a = parentHeaderValue === null || parentHeaderValue === void 0 ? void 0 : parentHeaderValue.session) !== null && _a !== void 0 ? _a : '',
            parentHeader: parentHeaderValue,
            content,
        });
        this._sendMessage(message);
    }
    /**
     * Send an `error` message to the client.
     *
     * @param parentHeader The parent header.
     * @param content The error content.
     */
    publishExecuteError(content, parentHeader = undefined) {
        var _a;
        const parentHeaderValue = typeof parentHeader !== 'undefined' ? parentHeader : this._parentHeader;
        const message = _jupyterlab_services__WEBPACK_IMPORTED_MODULE_0__.KernelMessage.createMessage({
            channel: 'iopub',
            msgType: 'error',
            // TODO: better handle this
            session: (_a = parentHeaderValue === null || parentHeaderValue === void 0 ? void 0 : parentHeaderValue.session) !== null && _a !== void 0 ? _a : '',
            parentHeader: parentHeaderValue,
            content,
        });
        this._sendMessage(message);
    }
    /**
     * Send a `update_display_data` message to the client.
     *
     * @param parentHeader The parent header.
     * @param content The update_display_data content.
     */
    updateDisplayData(content, parentHeader = undefined) {
        var _a;
        const parentHeaderValue = typeof parentHeader !== 'undefined' ? parentHeader : this._parentHeader;
        const message = _jupyterlab_services__WEBPACK_IMPORTED_MODULE_0__.KernelMessage.createMessage({
            channel: 'iopub',
            msgType: 'update_display_data',
            // TODO: better handle this
            session: (_a = parentHeaderValue === null || parentHeaderValue === void 0 ? void 0 : parentHeaderValue.session) !== null && _a !== void 0 ? _a : '',
            parentHeader: parentHeaderValue,
            content,
        });
        this._sendMessage(message);
    }
    /**
     * Send a `clear_output` message to the client.
     *
     * @param parentHeader The parent header.
     * @param content The clear_output content.
     */
    clearOutput(content, parentHeader = undefined) {
        var _a;
        const parentHeaderValue = typeof parentHeader !== 'undefined' ? parentHeader : this._parentHeader;
        const message = _jupyterlab_services__WEBPACK_IMPORTED_MODULE_0__.KernelMessage.createMessage({
            channel: 'iopub',
            msgType: 'clear_output',
            // TODO: better handle this
            session: (_a = parentHeaderValue === null || parentHeaderValue === void 0 ? void 0 : parentHeaderValue.session) !== null && _a !== void 0 ? _a : '',
            parentHeader: parentHeaderValue,
            content,
        });
        this._sendMessage(message);
    }
    /**
     * Send a `comm` message to the client.
     *
     * @param .
     */
    handleComm(type, content, metadata, buffers, parentHeader = undefined) {
        var _a;
        const parentHeaderValue = typeof parentHeader !== 'undefined' ? parentHeader : this._parentHeader;
        const message = _jupyterlab_services__WEBPACK_IMPORTED_MODULE_0__.KernelMessage.createMessage({
            channel: 'iopub',
            msgType: type,
            // TODO: better handle this
            session: (_a = parentHeaderValue === null || parentHeaderValue === void 0 ? void 0 : parentHeaderValue.session) !== null && _a !== void 0 ? _a : '',
            parentHeader: parentHeaderValue,
            content,
            metadata,
            buffers,
        });
        this._sendMessage(message);
    }
    /**
     * Send an 'idle' status message.
     *
     * @param parent The parent message
     */
    _idle(parent) {
        const message = _jupyterlab_services__WEBPACK_IMPORTED_MODULE_0__.KernelMessage.createMessage({
            msgType: 'status',
            session: parent.header.session,
            parentHeader: parent.header,
            channel: 'iopub',
            content: {
                execution_state: 'idle',
            },
        });
        this._sendMessage(message);
    }
    /**
     * Send a 'busy' status message.
     *
     * @param parent The parent message.
     */
    _busy(parent) {
        const message = _jupyterlab_services__WEBPACK_IMPORTED_MODULE_0__.KernelMessage.createMessage({
            msgType: 'status',
            session: parent.header.session,
            parentHeader: parent.header,
            channel: 'iopub',
            content: {
                execution_state: 'busy',
            },
        });
        this._sendMessage(message);
    }
    /**
     * Handle a kernel_info_request message
     *
     * @param parent The parent message.
     */
    async _kernelInfo(parent) {
        const content = await this.kernelInfoRequest();
        const message = _jupyterlab_services__WEBPACK_IMPORTED_MODULE_0__.KernelMessage.createMessage({
            msgType: 'kernel_info_reply',
            channel: 'shell',
            session: parent.header.session,
            parentHeader: parent.header,
            content,
        });
        this._sendMessage(message);
    }
    /**
     * Handle a `history_request` message
     *
     * @param msg The parent message.
     */
    async _historyRequest(msg) {
        const historyMsg = msg;
        const message = _jupyterlab_services__WEBPACK_IMPORTED_MODULE_0__.KernelMessage.createMessage({
            msgType: 'history_reply',
            channel: 'shell',
            parentHeader: historyMsg.header,
            session: msg.header.session,
            content: {
                status: 'ok',
                history: this._history,
            },
        });
        this._sendMessage(message);
    }
    /**
     * Send an `execute_input` message.
     *
     * @param msg The parent message.
     */
    _executeInput(msg) {
        const parent = msg;
        const code = parent.content.code;
        const message = _jupyterlab_services__WEBPACK_IMPORTED_MODULE_0__.KernelMessage.createMessage({
            msgType: 'execute_input',
            parentHeader: parent.header,
            channel: 'iopub',
            session: msg.header.session,
            content: {
                code,
                execution_count: this._executionCount,
            },
        });
        this._sendMessage(message);
    }
    /**
     * Handle an execute_request message.
     *
     * @param msg The parent message.
     */
    async _execute(msg) {
        const executeMsg = msg;
        const content = executeMsg.content;
        if (content.store_history) {
            this._executionCount++;
        }
        // TODO: handle differently
        this._parentHeader = executeMsg.header;
        this._executeInput(executeMsg);
        if (content.store_history) {
            this._history.push([0, 0, content.code]);
        }
        const reply = await this.executeRequest(executeMsg.content);
        const message = _jupyterlab_services__WEBPACK_IMPORTED_MODULE_0__.KernelMessage.createMessage({
            msgType: 'execute_reply',
            channel: 'shell',
            parentHeader: executeMsg.header,
            session: msg.header.session,
            content: reply,
        });
        this._sendMessage(message);
    }
    /**
     * Handle an complete_request message
     *
     * @param msg The parent message.
     */
    async _complete(msg) {
        const completeMsg = msg;
        const content = await this.completeRequest(completeMsg.content);
        const message = _jupyterlab_services__WEBPACK_IMPORTED_MODULE_0__.KernelMessage.createMessage({
            msgType: 'complete_reply',
            parentHeader: completeMsg.header,
            channel: 'shell',
            session: msg.header.session,
            content,
        });
        this._sendMessage(message);
    }
    /**
     * Handle an inspect_request message
     *
     * @param msg The parent message.
     */
    async _inspect(msg) {
        const inspectMsg = msg;
        const content = await this.inspectRequest(inspectMsg.content);
        const message = _jupyterlab_services__WEBPACK_IMPORTED_MODULE_0__.KernelMessage.createMessage({
            msgType: 'inspect_reply',
            parentHeader: inspectMsg.header,
            channel: 'shell',
            session: msg.header.session,
            content,
        });
        this._sendMessage(message);
    }
    /**
     * Handle an is_complete_request message
     *
     * @param msg The parent message.
     */
    async _isCompleteRequest(msg) {
        const isCompleteMsg = msg;
        const content = await this.isCompleteRequest(isCompleteMsg.content);
        const message = _jupyterlab_services__WEBPACK_IMPORTED_MODULE_0__.KernelMessage.createMessage({
            msgType: 'is_complete_reply',
            parentHeader: isCompleteMsg.header,
            channel: 'shell',
            session: msg.header.session,
            content,
        });
        this._sendMessage(message);
    }
}


/***/ }),

/***/ "../../../node_modules/@jupyterlite/pyodide-kernel/lib/_pypi.js":
/*!**********************************************************************!*\
  !*** ../../../node_modules/@jupyterlite/pyodide-kernel/lib/_pypi.js ***!
  \**********************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

var _pypi_all_json__WEBPACK_IMPORTED_MODULE_0___namespace_cache;
var _pypi_ipykernel_6_9_2_py3_none_any_whl__WEBPACK_IMPORTED_MODULE_1___namespace_cache;
var _pypi_piplite_0_4_6_py3_none_any_whl__WEBPACK_IMPORTED_MODULE_2___namespace_cache;
var _pypi_pyodide_kernel_0_4_6_py3_none_any_whl__WEBPACK_IMPORTED_MODULE_3___namespace_cache;
var _pypi_widgetsnbextension_3_6_999_py3_none_any_whl__WEBPACK_IMPORTED_MODULE_4___namespace_cache;
var _pypi_widgetsnbextension_4_0_999_py3_none_any_whl__WEBPACK_IMPORTED_MODULE_5___namespace_cache;
__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "allJSONUrl": () => (/* reexport fake namespace object from non-harmony */ _pypi_all_json__WEBPACK_IMPORTED_MODULE_0___namespace_cache || (_pypi_all_json__WEBPACK_IMPORTED_MODULE_0___namespace_cache = __webpack_require__.t(_pypi_all_json__WEBPACK_IMPORTED_MODULE_0__))),
/* harmony export */   "ipykernelWheelUrl": () => (/* reexport fake namespace object from non-harmony */ _pypi_ipykernel_6_9_2_py3_none_any_whl__WEBPACK_IMPORTED_MODULE_1___namespace_cache || (_pypi_ipykernel_6_9_2_py3_none_any_whl__WEBPACK_IMPORTED_MODULE_1___namespace_cache = __webpack_require__.t(_pypi_ipykernel_6_9_2_py3_none_any_whl__WEBPACK_IMPORTED_MODULE_1__))),
/* harmony export */   "pipliteWheelUrl": () => (/* reexport fake namespace object from non-harmony */ _pypi_piplite_0_4_6_py3_none_any_whl__WEBPACK_IMPORTED_MODULE_2___namespace_cache || (_pypi_piplite_0_4_6_py3_none_any_whl__WEBPACK_IMPORTED_MODULE_2___namespace_cache = __webpack_require__.t(_pypi_piplite_0_4_6_py3_none_any_whl__WEBPACK_IMPORTED_MODULE_2__))),
/* harmony export */   "pyodide_kernelWheelUrl": () => (/* reexport fake namespace object from non-harmony */ _pypi_pyodide_kernel_0_4_6_py3_none_any_whl__WEBPACK_IMPORTED_MODULE_3___namespace_cache || (_pypi_pyodide_kernel_0_4_6_py3_none_any_whl__WEBPACK_IMPORTED_MODULE_3___namespace_cache = __webpack_require__.t(_pypi_pyodide_kernel_0_4_6_py3_none_any_whl__WEBPACK_IMPORTED_MODULE_3__))),
/* harmony export */   "widgetsnbextensionWheelUrl": () => (/* reexport fake namespace object from non-harmony */ _pypi_widgetsnbextension_3_6_999_py3_none_any_whl__WEBPACK_IMPORTED_MODULE_4___namespace_cache || (_pypi_widgetsnbextension_3_6_999_py3_none_any_whl__WEBPACK_IMPORTED_MODULE_4___namespace_cache = __webpack_require__.t(_pypi_widgetsnbextension_3_6_999_py3_none_any_whl__WEBPACK_IMPORTED_MODULE_4__))),
/* harmony export */   "widgetsnbextensionWheelUrl1": () => (/* reexport fake namespace object from non-harmony */ _pypi_widgetsnbextension_4_0_999_py3_none_any_whl__WEBPACK_IMPORTED_MODULE_5___namespace_cache || (_pypi_widgetsnbextension_4_0_999_py3_none_any_whl__WEBPACK_IMPORTED_MODULE_5___namespace_cache = __webpack_require__.t(_pypi_widgetsnbextension_4_0_999_py3_none_any_whl__WEBPACK_IMPORTED_MODULE_5__)))
/* harmony export */ });
/* harmony import */ var _pypi_all_json__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../pypi/all.json */ "../../../node_modules/@jupyterlite/pyodide-kernel/pypi/all.json");
/* harmony import */ var _pypi_ipykernel_6_9_2_py3_none_any_whl__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../pypi/ipykernel-6.9.2-py3-none-any.whl */ "../../../node_modules/@jupyterlite/pyodide-kernel/pypi/ipykernel-6.9.2-py3-none-any.whl");
/* harmony import */ var _pypi_piplite_0_4_6_py3_none_any_whl__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../pypi/piplite-0.4.6-py3-none-any.whl */ "../../../node_modules/@jupyterlite/pyodide-kernel/pypi/piplite-0.4.6-py3-none-any.whl");
/* harmony import */ var _pypi_pyodide_kernel_0_4_6_py3_none_any_whl__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../pypi/pyodide_kernel-0.4.6-py3-none-any.whl */ "../../../node_modules/@jupyterlite/pyodide-kernel/pypi/pyodide_kernel-0.4.6-py3-none-any.whl");
/* harmony import */ var _pypi_widgetsnbextension_3_6_999_py3_none_any_whl__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../pypi/widgetsnbextension-3.6.999-py3-none-any.whl */ "../../../node_modules/@jupyterlite/pyodide-kernel/pypi/widgetsnbextension-3.6.999-py3-none-any.whl");
/* harmony import */ var _pypi_widgetsnbextension_4_0_999_py3_none_any_whl__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../pypi/widgetsnbextension-4.0.999-py3-none-any.whl */ "../../../node_modules/@jupyterlite/pyodide-kernel/pypi/widgetsnbextension-4.0.999-py3-none-any.whl");
// this file is autogenerated from the wheels described in ../package.json








/***/ }),

/***/ "../../../node_modules/@jupyterlite/pyodide-kernel/lib/index.js":
/*!**********************************************************************!*\
  !*** ../../../node_modules/@jupyterlite/pyodide-kernel/lib/index.js ***!
  \**********************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "PIPLITE_INDEX_SCHEMA": () => (/* reexport safe */ _tokens__WEBPACK_IMPORTED_MODULE_4__.PIPLITE_INDEX_SCHEMA),
/* harmony export */   "PyodideCoincidentKernel": () => (/* reexport safe */ _coincident_worker__WEBPACK_IMPORTED_MODULE_1__.PyodideCoincidentKernel),
/* harmony export */   "PyodideComlinkKernel": () => (/* reexport safe */ _comlink_worker__WEBPACK_IMPORTED_MODULE_2__.PyodideComlinkKernel),
/* harmony export */   "PyodideKernel": () => (/* reexport safe */ _kernel__WEBPACK_IMPORTED_MODULE_3__.PyodideKernel),
/* harmony export */   "PyodideRemoteKernel": () => (/* reexport safe */ _worker__WEBPACK_IMPORTED_MODULE_5__.PyodideRemoteKernel),
/* harmony export */   "SharedBufferContentsAPI": () => (/* reexport safe */ _coincident_worker__WEBPACK_IMPORTED_MODULE_1__.SharedBufferContentsAPI),
/* harmony export */   "allJSONUrl": () => (/* reexport safe */ _pypi__WEBPACK_IMPORTED_MODULE_0__.allJSONUrl),
/* harmony export */   "ipykernelWheelUrl": () => (/* reexport safe */ _pypi__WEBPACK_IMPORTED_MODULE_0__.ipykernelWheelUrl),
/* harmony export */   "pipliteWheelUrl": () => (/* reexport safe */ _pypi__WEBPACK_IMPORTED_MODULE_0__.pipliteWheelUrl),
/* harmony export */   "pyodide_kernelWheelUrl": () => (/* reexport safe */ _pypi__WEBPACK_IMPORTED_MODULE_0__.pyodide_kernelWheelUrl),
/* harmony export */   "widgetsnbextensionWheelUrl": () => (/* reexport safe */ _pypi__WEBPACK_IMPORTED_MODULE_0__.widgetsnbextensionWheelUrl),
/* harmony export */   "widgetsnbextensionWheelUrl1": () => (/* reexport safe */ _pypi__WEBPACK_IMPORTED_MODULE_0__.widgetsnbextensionWheelUrl1)
/* harmony export */ });
/* harmony import */ var _pypi__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./_pypi */ "../../../node_modules/@jupyterlite/pyodide-kernel/lib/_pypi.js");
/* harmony import */ var _coincident_worker__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./coincident.worker */ "../../../node_modules/@jupyterlite/pyodide-kernel/lib/coincident.worker.js");
/* harmony import */ var _comlink_worker__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./comlink.worker */ "../../../node_modules/@jupyterlite/pyodide-kernel/lib/comlink.worker.js");
/* harmony import */ var _kernel__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./kernel */ "../../../node_modules/@jupyterlite/pyodide-kernel/lib/kernel.js");
/* harmony import */ var _tokens__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./tokens */ "../../../node_modules/@jupyterlite/pyodide-kernel/lib/tokens.js");
/* harmony import */ var _worker__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./worker */ "../../../node_modules/@jupyterlite/pyodide-kernel/lib/worker.js");
// Copyright (c) Jupyter Development Team.
// Distributed under the terms of the Modified BSD License.








/***/ }),

/***/ "../../../node_modules/@jupyterlite/pyodide-kernel/lib/kernel.js":
/*!***********************************************************************!*\
  !*** ../../../node_modules/@jupyterlite/pyodide-kernel/lib/kernel.js ***!
  \***********************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "PyodideKernel": () => (/* binding */ PyodideKernel)
/* harmony export */ });
/* harmony import */ var coincident__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! coincident */ "../../../node_modules/coincident/esm/index.js");
/* harmony import */ var comlink__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! comlink */ "../../../node_modules/comlink/dist/esm/comlink.mjs");
/* harmony import */ var _lumino_coreutils__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @lumino/coreutils */ "webpack/sharing/consume/default/@lumino/coreutils");
/* harmony import */ var _lumino_coreutils__WEBPACK_IMPORTED_MODULE_1___default = /*#__PURE__*/__webpack_require__.n(_lumino_coreutils__WEBPACK_IMPORTED_MODULE_1__);
/* harmony import */ var _jupyterlab_coreutils__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! @jupyterlab/coreutils */ "webpack/sharing/consume/default/@jupyterlab/coreutils");
/* harmony import */ var _jupyterlab_coreutils__WEBPACK_IMPORTED_MODULE_2___default = /*#__PURE__*/__webpack_require__.n(_jupyterlab_coreutils__WEBPACK_IMPORTED_MODULE_2__);
/* harmony import */ var _jupyterlite_kernel__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @jupyterlite/kernel */ "../../../node_modules/@jupyterlite/kernel/lib/kernel.js");
/* harmony import */ var _pypi__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ./_pypi */ "../../../node_modules/@jupyterlite/pyodide-kernel/pypi/all.json");
/* harmony import */ var _pypi__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ./_pypi */ "../../../node_modules/@jupyterlite/pyodide-kernel/pypi/piplite-0.4.6-py3-none-any.whl");
/* harmony import */ var _jupyterlite_contents__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! @jupyterlite/contents */ "../../../node_modules/@jupyterlite/pyodide-kernel/node_modules/@jupyterlite/contents/lib/drivecontents.js");







/**
 * A kernel that executes Python code with Pyodide.
 */
class PyodideKernel extends _jupyterlite_kernel__WEBPACK_IMPORTED_MODULE_3__.BaseKernel {
    /**
     * Instantiate a new PyodideKernel
     *
     * @param options The instantiation options for a new PyodideKernel
     */
    constructor(options) {
        super(options);
        this._ready = new _lumino_coreutils__WEBPACK_IMPORTED_MODULE_1__.PromiseDelegate();
        this._worker = this.initWorker(options);
        this._remoteKernel = this.initRemote(options);
        this._contentsManager = options.contentsManager;
    }
    /**
     * Load the worker.
     *
     * ### Note
     *
     * Subclasses must implement this typographically almost _exactly_ for
     * webpack to find it.
     */
    initWorker(options) {
        if (crossOriginIsolated) {
            return new Worker(new URL(/* worker import */ __webpack_require__.p + __webpack_require__.u("node_modules_process_browser_js-node_modules_jupyterlite_pyodide-kernel_lib_lazy_recursive-_4-9e35290"), __webpack_require__.b), {
                type: undefined,
            });
        }
        else {
            return new Worker(new URL(/* worker import */ __webpack_require__.p + __webpack_require__.u("node_modules_process_browser_js-node_modules_jupyterlite_pyodide-kernel_lib_lazy_recursive-_4-9e35291"), __webpack_require__.b), {
                type: undefined,
            });
        }
    }
    /**
     * Initialize the remote kernel.
     * Use coincident if crossOriginIsolated, comlink otherwise
     * See the two following issues for more context:
     *  - https://github.com/jupyterlite/jupyterlite/issues/1424
     *  - https://github.com/jupyterlite/pyodide-kernel/pull/126
     */
    initRemote(options) {
        let remote;
        if (crossOriginIsolated) {
            remote = (0,coincident__WEBPACK_IMPORTED_MODULE_0__["default"])(this._worker);
            remote.processWorkerMessage = this._processWorkerMessage.bind(this);
            // The coincident worker uses its own filesystem API:
            remote.processDriveRequest = async (data) => {
                if (!_jupyterlite_contents__WEBPACK_IMPORTED_MODULE_4__.DriveContentsProcessor) {
                    throw new Error('File system calls over Atomics.wait is only supported with jupyterlite>=0.4.0a3');
                }
                if (this._contentsProcessor === undefined) {
                    this._contentsProcessor = new _jupyterlite_contents__WEBPACK_IMPORTED_MODULE_4__.DriveContentsProcessor({
                        contentsManager: this._contentsManager,
                    });
                }
                return await this._contentsProcessor.processDriveRequest(data);
            };
        }
        else {
            remote = (0,comlink__WEBPACK_IMPORTED_MODULE_5__.wrap)(this._worker);
            // we use the normal postMessage mechanism
            this._worker.addEventListener('message', (ev) => {
                var _a;
                if (typeof ((_a = ev === null || ev === void 0 ? void 0 : ev.data) === null || _a === void 0 ? void 0 : _a._kernelMessage) !== 'undefined') {
                    // only process non comlink messages
                    this._processWorkerMessage(ev.data._kernelMessage);
                }
            });
        }
        const remoteOptions = this.initRemoteOptions(options);
        remote.initialize(remoteOptions).then(this._ready.resolve.bind(this._ready));
        return remote;
    }
    initRemoteOptions(options) {
        const { pyodideUrl } = options;
        const indexUrl = pyodideUrl.slice(0, pyodideUrl.lastIndexOf('/') + 1);
        const baseUrl = _jupyterlab_coreutils__WEBPACK_IMPORTED_MODULE_2__.PageConfig.getBaseUrl();
        const pipliteUrls = [...(options.pipliteUrls || []), _pypi__WEBPACK_IMPORTED_MODULE_6__];
        const disablePyPIFallback = !!options.disablePyPIFallback;
        return {
            baseUrl,
            pyodideUrl,
            indexUrl,
            pipliteWheelUrl: options.pipliteWheelUrl || _pypi__WEBPACK_IMPORTED_MODULE_7__,
            pipliteUrls,
            disablePyPIFallback,
            location: this.location,
            mountDrive: options.mountDrive,
            loadPyodideOptions: options.loadPyodideOptions || {},
        };
    }
    /**
     * Dispose the kernel.
     */
    dispose() {
        if (this.isDisposed) {
            return;
        }
        this._worker.terminate();
        this._worker = null;
        super.dispose();
    }
    /**
     * A promise that is fulfilled when the kernel is ready.
     */
    get ready() {
        return this._ready.promise;
    }
    /**
     * Process a message coming from the pyodide web worker.
     *
     * @param msg The worker message to process.
     */
    _processWorkerMessage(msg) {
        var _a, _b, _c, _d, _e, _f, _g;
        if (!msg.type) {
            return;
        }
        switch (msg.type) {
            case 'stream': {
                const bundle = (_a = msg.bundle) !== null && _a !== void 0 ? _a : { name: 'stdout', text: '' };
                this.stream(bundle, msg.parentHeader);
                break;
            }
            case 'input_request': {
                const bundle = (_b = msg.content) !== null && _b !== void 0 ? _b : { prompt: '', password: false };
                this.inputRequest(bundle, msg.parentHeader);
                break;
            }
            case 'display_data': {
                const bundle = (_c = msg.bundle) !== null && _c !== void 0 ? _c : { data: {}, metadata: {}, transient: {} };
                this.displayData(bundle, msg.parentHeader);
                break;
            }
            case 'update_display_data': {
                const bundle = (_d = msg.bundle) !== null && _d !== void 0 ? _d : { data: {}, metadata: {}, transient: {} };
                this.updateDisplayData(bundle, msg.parentHeader);
                break;
            }
            case 'clear_output': {
                const bundle = (_e = msg.bundle) !== null && _e !== void 0 ? _e : { wait: false };
                this.clearOutput(bundle, msg.parentHeader);
                break;
            }
            case 'execute_result': {
                const bundle = (_f = msg.bundle) !== null && _f !== void 0 ? _f : {
                    execution_count: 0,
                    data: {},
                    metadata: {},
                };
                this.publishExecuteResult(bundle, msg.parentHeader);
                break;
            }
            case 'execute_error': {
                const bundle = (_g = msg.bundle) !== null && _g !== void 0 ? _g : { ename: '', evalue: '', traceback: [] };
                this.publishExecuteError(bundle, msg.parentHeader);
                break;
            }
            case 'comm_msg':
            case 'comm_open':
            case 'comm_close': {
                this.handleComm(msg.type, msg.content, msg.metadata, msg.buffers, msg.parentHeader);
                break;
            }
        }
    }
    /**
     * Handle a kernel_info_request message
     */
    async kernelInfoRequest() {
        const content = {
            implementation: 'pyodide',
            implementation_version: '0.1.0',
            language_info: {
                codemirror_mode: {
                    name: 'python',
                    version: 3,
                },
                file_extension: '.py',
                mimetype: 'text/x-python',
                name: 'python',
                nbconvert_exporter: 'python',
                pygments_lexer: 'ipython3',
                version: '3.8',
            },
            protocol_version: '5.3',
            status: 'ok',
            banner: 'A WebAssembly-powered Python kernel backed by Pyodide',
            help_links: [
                {
                    text: 'Python (WASM) Kernel',
                    url: 'https://pyodide.org',
                },
            ],
        };
        return content;
    }
    /**
     * Handle an `execute_request` message
     *
     * @param msg The parent message.
     */
    async executeRequest(content) {
        await this.ready;
        const result = await this._remoteKernel.execute(content, this.parent);
        result.execution_count = this.executionCount;
        return result;
    }
    /**
     * Handle an complete_request message
     *
     * @param msg The parent message.
     */
    async completeRequest(content) {
        return await this._remoteKernel.complete(content, this.parent);
    }
    /**
     * Handle an `inspect_request` message.
     *
     * @param content - The content of the request.
     *
     * @returns A promise that resolves with the response message.
     */
    async inspectRequest(content) {
        return await this._remoteKernel.inspect(content, this.parent);
    }
    /**
     * Handle an `is_complete_request` message.
     *
     * @param content - The content of the request.
     *
     * @returns A promise that resolves with the response message.
     */
    async isCompleteRequest(content) {
        return await this._remoteKernel.isComplete(content, this.parent);
    }
    /**
     * Handle a `comm_info_request` message.
     *
     * @param content - The content of the request.
     *
     * @returns A promise that resolves with the response message.
     */
    async commInfoRequest(content) {
        return await this._remoteKernel.commInfo(content, this.parent);
    }
    /**
     * Send an `comm_open` message.
     *
     * @param msg - The comm_open message.
     */
    async commOpen(msg) {
        return await this._remoteKernel.commOpen(msg, this.parent);
    }
    /**
     * Send an `comm_msg` message.
     *
     * @param msg - The comm_msg message.
     */
    async commMsg(msg) {
        return await this._remoteKernel.commMsg(msg, this.parent);
    }
    /**
     * Send an `comm_close` message.
     *
     * @param close - The comm_close message.
     */
    async commClose(msg) {
        return await this._remoteKernel.commClose(msg, this.parent);
    }
    /**
     * Send an `input_reply` message.
     *
     * @param content - The content of the reply.
     */
    async inputReply(content) {
        return await this._remoteKernel.inputReply(content, this.parent);
    }
}


/***/ }),

/***/ "../../../node_modules/@jupyterlite/pyodide-kernel/lib/tokens.js":
/*!***********************************************************************!*\
  !*** ../../../node_modules/@jupyterlite/pyodide-kernel/lib/tokens.js ***!
  \***********************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

var _schema_piplite_v0_schema_json__WEBPACK_IMPORTED_MODULE_0___namespace_cache;
__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "PIPLITE_INDEX_SCHEMA": () => (/* reexport fake namespace object from non-harmony */ _schema_piplite_v0_schema_json__WEBPACK_IMPORTED_MODULE_0___namespace_cache || (_schema_piplite_v0_schema_json__WEBPACK_IMPORTED_MODULE_0___namespace_cache = __webpack_require__.t(_schema_piplite_v0_schema_json__WEBPACK_IMPORTED_MODULE_0__, 2)))
/* harmony export */ });
/* harmony import */ var _schema_piplite_v0_schema_json__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../schema/piplite.v0.schema.json */ "../../../node_modules/@jupyterlite/pyodide-kernel/schema/piplite.v0.schema.json");
// Copyright (c) Jupyter Development Team.
// Distributed under the terms of the Modified BSD License.
/**
 * The schema for a Warehouse-like index, as used by piplite.
 */



/***/ }),

/***/ "../../../node_modules/@jupyterlite/pyodide-kernel/lib/worker.js":
/*!***********************************************************************!*\
  !*** ../../../node_modules/@jupyterlite/pyodide-kernel/lib/worker.js ***!
  \***********************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "PyodideRemoteKernel": () => (/* binding */ PyodideRemoteKernel)
/* harmony export */ });
// Copyright (c) Jupyter Development Team.
// Distributed under the terms of the Modified BSD License.
class PyodideRemoteKernel {
    constructor() {
        /**
         * Initialization options.
         */
        this._options = null;
        this._initializer = null;
        this._pyodide = null;
        /** TODO: real typing */
        this._localPath = '';
        this._driveName = '';
        this._driveFS = null;
        this._sendWorkerMessage = () => { };
        this._initialized = new Promise((resolve, reject) => {
            this._initializer = { resolve, reject };
        });
    }
    /**
     * Accept the URLs from the host
     **/
    async initialize(options) {
        var _a;
        this._options = options;
        if (options.location.includes(':')) {
            const parts = options.location.split(':');
            this._driveName = parts[0];
            this._localPath = parts[1];
        }
        else {
            this._driveName = '';
            this._localPath = options.location;
        }
        await this.initRuntime(options);
        await this.initFilesystem(options);
        await this.initPackageManager(options);
        await this.initKernel(options);
        await this.initGlobals(options);
        (_a = this._initializer) === null || _a === void 0 ? void 0 : _a.resolve();
    }
    async initRuntime(options) {
        const { pyodideUrl, indexUrl } = options;
        let loadPyodide;
        if (pyodideUrl.endsWith('.mjs')) {
            // note: this does not work at all in firefox
            const pyodideModule = await import(
            /* webpackIgnore: true */ pyodideUrl);
            loadPyodide = pyodideModule.loadPyodide;
        }
        else {
            importScripts(pyodideUrl);
            loadPyodide = self.loadPyodide;
        }
        this._pyodide = await loadPyodide({
            indexURL: indexUrl,
            ...options.loadPyodideOptions,
        });
    }
    async initPackageManager(options) {
        if (!this._options) {
            throw new Error('Uninitialized');
        }
        const { pipliteWheelUrl, disablePyPIFallback, pipliteUrls, loadPyodideOptions } = this._options;
        const preloaded = (loadPyodideOptions || {}).packages || [];
        if (!preloaded.includes('micropip')) {
            await this._pyodide.loadPackage(['micropip']);
        }
        if (!preloaded.includes('piplite')) {
            await this._pyodide.runPythonAsync(`
      import micropip
      await micropip.install('${pipliteWheelUrl}', keep_going=True)
    `);
        }
        // get piplite early enough to impact pyodide-kernel dependencies
        await this._pyodide.runPythonAsync(`
      import piplite.piplite
      piplite.piplite._PIPLITE_DISABLE_PYPI = ${disablePyPIFallback ? 'True' : 'False'}
      piplite.piplite._PIPLITE_URLS = ${JSON.stringify(pipliteUrls)}
    `);
    }
    async initKernel(options) {
        const preloaded = (options.loadPyodideOptions || {}).packages || [];
        const toLoad = ['ssl', 'sqlite3', 'ipykernel', 'comm', 'pyodide_kernel', 'ipython'];
        const scriptLines = [];
        // use piplite for packages that weren't pre-loaded
        for (const pkgName of toLoad) {
            if (!preloaded.includes(pkgName)) {
                scriptLines.push(`await piplite.install('${pkgName}', keep_going=True)`);
            }
        }
        // import the kernel
        scriptLines.push('import pyodide_kernel');
        // cd to the kernel location
        if (options.mountDrive && this._localPath) {
            scriptLines.push('import os', `os.chdir("${this._localPath}")`);
        }
        // from this point forward, only use piplite (but not %pip)
        await this._pyodide.runPythonAsync(scriptLines.join('\n'));
    }
    async initGlobals(options) {
        const { globals } = this._pyodide;
        this._kernel = globals.get('pyodide_kernel').kernel_instance.copy();
        this._stdout_stream = globals.get('pyodide_kernel').stdout_stream.copy();
        this._stderr_stream = globals.get('pyodide_kernel').stderr_stream.copy();
        this._interpreter = this._kernel.interpreter.copy();
        this._interpreter.send_comm = this.sendComm.bind(this);
    }
    /**
     * Setup custom Emscripten FileSystem
     */
    async initFilesystem(options) {
        if (options.mountDrive) {
            const mountpoint = '/drive';
            const { FS, PATH, ERRNO_CODES } = this._pyodide;
            const { baseUrl } = options;
            const { DriveFS } = await __webpack_require__.e(/*! import() */ "vendors-node_modules_jupyterlite_pyodide-kernel_node_modules_jupyterlite_contents_lib_index_js").then(__webpack_require__.bind(__webpack_require__, /*! @jupyterlite/contents */ "../../../node_modules/@jupyterlite/pyodide-kernel/node_modules/@jupyterlite/contents/lib/index.js"));
            const driveFS = new DriveFS({
                FS,
                PATH,
                ERRNO_CODES,
                baseUrl,
                driveName: this._driveName,
                mountpoint,
            });
            FS.mkdir(mountpoint);
            FS.mount(driveFS, {}, mountpoint);
            FS.chdir(mountpoint);
            this._driveFS = driveFS;
        }
    }
    /**
     * Recursively convert a Map to a JavaScript object
     * @param obj A Map, Array, or other  object to convert
     */
    mapToObject(obj) {
        const out = obj instanceof Array ? [] : {};
        obj.forEach((value, key) => {
            out[key] =
                value instanceof Map || value instanceof Array
                    ? this.mapToObject(value)
                    : value;
        });
        return out;
    }
    /**
     * Format the response from the Pyodide evaluation.
     *
     * @param res The result object from the Pyodide evaluation
     */
    formatResult(res) {
        if (!(res instanceof this._pyodide.ffi.PyProxy)) {
            return res;
        }
        // TODO: this is a bit brittle
        const m = res.toJs();
        const results = this.mapToObject(m);
        return results;
    }
    /**
     * Register the callback function to send messages from the worker back to the main thread.
     * @param callback the callback to register
     */
    registerCallback(callback) {
        this._sendWorkerMessage = callback;
    }
    /**
     * Makes sure pyodide is ready before continuing, and cache the parent message.
     */
    async setup(parent) {
        await this._initialized;
        this._kernel._parent_header = this._pyodide.toPy(parent);
    }
    /**
     * Execute code with the interpreter.
     *
     * @param content The incoming message with the code to execute.
     */
    async execute(content, parent) {
        await this.setup(parent);
        const publishExecutionResult = (prompt_count, data, metadata) => {
            const bundle = {
                execution_count: prompt_count,
                data: this.formatResult(data),
                metadata: this.formatResult(metadata),
            };
            this._sendWorkerMessage({
                parentHeader: this.formatResult(this._kernel._parent_header)['header'],
                bundle,
                type: 'execute_result',
            });
        };
        const publishExecutionError = (ename, evalue, traceback) => {
            const bundle = {
                ename: ename,
                evalue: evalue,
                traceback: traceback,
            };
            this._sendWorkerMessage({
                parentHeader: this.formatResult(this._kernel._parent_header)['header'],
                bundle,
                type: 'execute_error',
            });
        };
        const clearOutputCallback = (wait) => {
            const bundle = {
                wait: this.formatResult(wait),
            };
            this._sendWorkerMessage({
                parentHeader: this.formatResult(this._kernel._parent_header)['header'],
                bundle,
                type: 'clear_output',
            });
        };
        const displayDataCallback = (data, metadata, transient) => {
            const bundle = {
                data: this.formatResult(data),
                metadata: this.formatResult(metadata),
                transient: this.formatResult(transient),
            };
            this._sendWorkerMessage({
                parentHeader: this.formatResult(this._kernel._parent_header)['header'],
                bundle,
                type: 'display_data',
            });
        };
        const updateDisplayDataCallback = (data, metadata, transient) => {
            const bundle = {
                data: this.formatResult(data),
                metadata: this.formatResult(metadata),
                transient: this.formatResult(transient),
            };
            this._sendWorkerMessage({
                parentHeader: this.formatResult(this._kernel._parent_header)['header'],
                bundle,
                type: 'update_display_data',
            });
        };
        const publishStreamCallback = (name, text) => {
            const bundle = {
                name: this.formatResult(name),
                text: this.formatResult(text),
            };
            this._sendWorkerMessage({
                parentHeader: this.formatResult(this._kernel._parent_header)['header'],
                bundle,
                type: 'stream',
            });
        };
        this._stdout_stream.publish_stream_callback = publishStreamCallback;
        this._stderr_stream.publish_stream_callback = publishStreamCallback;
        this._interpreter.display_pub.clear_output_callback = clearOutputCallback;
        this._interpreter.display_pub.display_data_callback = displayDataCallback;
        this._interpreter.display_pub.update_display_data_callback =
            updateDisplayDataCallback;
        this._interpreter.displayhook.publish_execution_result = publishExecutionResult;
        this._interpreter.input = this.input.bind(this);
        this._interpreter.getpass = this.getpass.bind(this);
        const res = await this._kernel.run(content.code);
        const results = this.formatResult(res);
        if (results['status'] === 'error') {
            publishExecutionError(results['ename'], results['evalue'], results['traceback']);
        }
        return results;
    }
    /**
     * Complete the code submitted by a user.
     *
     * @param content The incoming message with the code to complete.
     */
    async complete(content, parent) {
        await this.setup(parent);
        const res = this._kernel.complete(content.code, content.cursor_pos);
        const results = this.formatResult(res);
        return results;
    }
    /**
     * Inspect the code submitted by a user.
     *
     * @param content The incoming message with the code to inspect.
     */
    async inspect(content, parent) {
        await this.setup(parent);
        const res = this._kernel.inspect(content.code, content.cursor_pos, content.detail_level);
        const results = this.formatResult(res);
        return results;
    }
    /**
     * Check code for completeness submitted by a user.
     *
     * @param content The incoming message with the code to check.
     */
    async isComplete(content, parent) {
        await this.setup(parent);
        const res = this._kernel.is_complete(content.code);
        const results = this.formatResult(res);
        return results;
    }
    /**
     * Respond to the commInfoRequest.
     *
     * @param content The incoming message with the comm target name.
     */
    async commInfo(content, parent) {
        await this.setup(parent);
        const res = this._kernel.comm_info(content.target_name);
        const results = this.formatResult(res);
        return {
            comms: results,
            status: 'ok',
        };
    }
    /**
     * Respond to the commOpen.
     *
     * @param content The incoming message with the comm open.
     */
    async commOpen(content, parent) {
        await this.setup(parent);
        const res = this._kernel.comm_manager.comm_open(this._pyodide.toPy(null), this._pyodide.toPy(null), this._pyodide.toPy(content));
        const results = this.formatResult(res);
        return results;
    }
    /**
     * Respond to the commMsg.
     *
     * @param content The incoming message with the comm msg.
     */
    async commMsg(content, parent) {
        await this.setup(parent);
        const res = this._kernel.comm_manager.comm_msg(this._pyodide.toPy(null), this._pyodide.toPy(null), this._pyodide.toPy(content));
        const results = this.formatResult(res);
        return results;
    }
    /**
     * Respond to the commClose.
     *
     * @param content The incoming message with the comm close.
     */
    async commClose(content, parent) {
        await this.setup(parent);
        const res = this._kernel.comm_manager.comm_close(this._pyodide.toPy(null), this._pyodide.toPy(null), this._pyodide.toPy(content));
        const results = this.formatResult(res);
        return results;
    }
    /**
     * Resolve the input request by getting back the reply from the main thread
     *
     * @param content The incoming message with the reply
     */
    async inputReply(content, parent) {
        await this.setup(parent);
        this._resolveInputReply(content);
    }
    /**
     * Send a input request to the front-end.
     *
     * @param prompt the text to show at the prompt
     * @param password Is the request for a password?
     */
    async sendInputRequest(prompt, password) {
        const content = {
            prompt,
            password,
        };
        this._sendWorkerMessage({
            type: 'input_request',
            parentHeader: this.formatResult(this._kernel._parent_header)['header'],
            content,
        });
    }
    async getpass(prompt) {
        prompt = typeof prompt === 'undefined' ? '' : prompt;
        await this.sendInputRequest(prompt, true);
        const replyPromise = new Promise((resolve) => {
            this._resolveInputReply = resolve;
        });
        const result = await replyPromise;
        return result['value'];
    }
    async input(prompt) {
        prompt = typeof prompt === 'undefined' ? '' : prompt;
        await this.sendInputRequest(prompt, false);
        const replyPromise = new Promise((resolve) => {
            this._resolveInputReply = resolve;
        });
        const result = await replyPromise;
        return result['value'];
    }
    /**
     * Send a comm message to the front-end.
     *
     * @param type The type of the comm message.
     * @param content The content.
     * @param metadata The metadata.
     * @param ident The ident.
     * @param buffers The binary buffers.
     */
    async sendComm(type, content, metadata, ident, buffers) {
        this._sendWorkerMessage({
            type: type,
            content: this.formatResult(content),
            metadata: this.formatResult(metadata),
            ident: this.formatResult(ident),
            buffers: this.formatResult(buffers),
            parentHeader: this.formatResult(this._kernel._parent_header)['header'],
        });
    }
}


/***/ }),

/***/ "../../../node_modules/@jupyterlite/pyodide-kernel/node_modules/@jupyterlite/contents/lib/drivecontents.js":
/*!*****************************************************************************************************************!*\
  !*** ../../../node_modules/@jupyterlite/pyodide-kernel/node_modules/@jupyterlite/contents/lib/drivecontents.js ***!
  \*****************************************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "DriveContentsProcessor": () => (/* binding */ DriveContentsProcessor)
/* harmony export */ });
/* harmony import */ var _jupyterlab_coreutils__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @jupyterlab/coreutils */ "webpack/sharing/consume/default/@jupyterlab/coreutils");
/* harmony import */ var _jupyterlab_coreutils__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(_jupyterlab_coreutils__WEBPACK_IMPORTED_MODULE_0__);
/* harmony import */ var _drivefs__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./drivefs */ "../../../node_modules/@jupyterlite/pyodide-kernel/node_modules/@jupyterlite/contents/lib/drivefs.js");
/* harmony import */ var _emscripten__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./emscripten */ "../../../node_modules/@jupyterlite/pyodide-kernel/node_modules/@jupyterlite/contents/lib/emscripten.js");



/**
 * Class for processing a drive request from the DriveFS.
 */
class DriveContentsProcessor {
    constructor(options) {
        this.contentsManager = options.contentsManager;
    }
    async processDriveRequest(request) {
        switch (request.method) {
            case 'readdir':
                return this.readdir(request);
            case 'rmdir':
                return this.rmdir(request);
            case 'rename':
                return this.rename(request);
            case 'getmode':
                return this.getmode(request);
            case 'lookup':
                return this.lookup(request);
            case 'mknod':
                return this.mknod(request);
            case 'getattr':
                return this.getattr(request);
            case 'get':
                return this.get(request);
            case 'put':
                return this.put(request);
        }
        throw `Drive request ${request.method} does not exist.`;
    }
    async readdir(request) {
        const model = await this.contentsManager.get(request.path, { content: true });
        let response = [];
        if (model.type === 'directory' && model.content) {
            response = model.content.map((subcontent) => subcontent.name);
        }
        return response;
    }
    async rmdir(request) {
        await this.contentsManager.delete(request.path);
        return null;
    }
    async rename(request) {
        await this.contentsManager.rename(request.path, request.data.newPath);
        return null;
    }
    async getmode(request) {
        const model = await this.contentsManager.get(request.path);
        let response;
        if (model.type === 'directory') {
            response = _emscripten__WEBPACK_IMPORTED_MODULE_1__.DIR_MODE;
        }
        else {
            response = _emscripten__WEBPACK_IMPORTED_MODULE_1__.FILE_MODE;
        }
        return response;
    }
    async lookup(request) {
        let response;
        try {
            const model = await this.contentsManager.get(request.path);
            response = {
                ok: true,
                mode: model.type === 'directory' ? _emscripten__WEBPACK_IMPORTED_MODULE_1__.DIR_MODE : _emscripten__WEBPACK_IMPORTED_MODULE_1__.FILE_MODE,
            };
        }
        catch (e) {
            response = { ok: false };
        }
        return response;
    }
    async mknod(request) {
        const model = await this.contentsManager.newUntitled({
            path: _jupyterlab_coreutils__WEBPACK_IMPORTED_MODULE_0__.PathExt.dirname(request.path),
            type: request.data.mode === _emscripten__WEBPACK_IMPORTED_MODULE_1__.DIR_MODE ? 'directory' : 'file',
            ext: _jupyterlab_coreutils__WEBPACK_IMPORTED_MODULE_0__.PathExt.extname(request.path),
        });
        await this.contentsManager.rename(model.path, request.path);
        return null;
    }
    async getattr(request) {
        const model = await this.contentsManager.get(request.path);
        // create a default date for drives that send incomplete information
        // for nested foldes and files
        const defaultDate = new Date(0).toISOString();
        return {
            dev: 1,
            nlink: 1,
            uid: 0,
            gid: 0,
            rdev: 0,
            size: model.size || 0,
            blksize: _drivefs__WEBPACK_IMPORTED_MODULE_2__.BLOCK_SIZE,
            blocks: Math.ceil(model.size || 0 / _drivefs__WEBPACK_IMPORTED_MODULE_2__.BLOCK_SIZE),
            atime: model.last_modified || defaultDate,
            mtime: model.last_modified || defaultDate,
            ctime: model.created || defaultDate,
            timestamp: 0,
        };
    }
    async get(request) {
        const model = await this.contentsManager.get(request.path, { content: true });
        let response;
        if (model.type !== 'directory') {
            response = {
                content: model.format === 'json' ? JSON.stringify(model.content) : model.content,
                format: model.format,
            };
        }
        return response;
    }
    async put(request) {
        await this.contentsManager.save(request.path, {
            content: request.data.format === 'json'
                ? JSON.parse(request.data.data)
                : request.data.data,
            type: 'file',
            format: request.data.format,
        });
        return null;
    }
}


/***/ }),

/***/ "../../../node_modules/@jupyterlite/pyodide-kernel/node_modules/@jupyterlite/contents/lib/drivefs.js":
/*!***********************************************************************************************************!*\
  !*** ../../../node_modules/@jupyterlite/pyodide-kernel/node_modules/@jupyterlite/contents/lib/drivefs.js ***!
  \***********************************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "BLOCK_SIZE": () => (/* binding */ BLOCK_SIZE),
/* harmony export */   "ContentsAPI": () => (/* binding */ ContentsAPI),
/* harmony export */   "DRIVE_API_PATH": () => (/* binding */ DRIVE_API_PATH),
/* harmony export */   "DRIVE_SEPARATOR": () => (/* binding */ DRIVE_SEPARATOR),
/* harmony export */   "DriveFS": () => (/* binding */ DriveFS),
/* harmony export */   "DriveFSEmscriptenNodeOps": () => (/* binding */ DriveFSEmscriptenNodeOps),
/* harmony export */   "DriveFSEmscriptenStreamOps": () => (/* binding */ DriveFSEmscriptenStreamOps),
/* harmony export */   "ServiceWorkerContentsAPI": () => (/* binding */ ServiceWorkerContentsAPI)
/* harmony export */ });
/* harmony import */ var _emscripten__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./emscripten */ "../../../node_modules/@jupyterlite/pyodide-kernel/node_modules/@jupyterlite/contents/lib/emscripten.js");
// Copyright (c) Jupyter Development Team.
// Distributed under the terms of the Modified BSD License.

const DRIVE_SEPARATOR = ':';
const DRIVE_API_PATH = '/api/drive.v1';
const BLOCK_SIZE = 4096;
const encoder = new TextEncoder();
const decoder = new TextDecoder('utf-8');
// Mapping flag -> do we need to overwrite the file upon closing it
const flagNeedsWrite = {
    0 /*O_RDONLY*/: false,
    1 /*O_WRONLY*/: true,
    2 /*O_RDWR*/: true,
    64 /*O_CREAT*/: true,
    65 /*O_WRONLY|O_CREAT*/: true,
    66 /*O_RDWR|O_CREAT*/: true,
    129 /*O_WRONLY|O_EXCL*/: true,
    193 /*O_WRONLY|O_CREAT|O_EXCL*/: true,
    514 /*O_RDWR|O_TRUNC*/: true,
    577 /*O_WRONLY|O_CREAT|O_TRUNC*/: true,
    578 /*O_CREAT|O_RDWR|O_TRUNC*/: true,
    705 /*O_WRONLY|O_CREAT|O_EXCL|O_TRUNC*/: true,
    706 /*O_RDWR|O_CREAT|O_EXCL|O_TRUNC*/: true,
    1024 /*O_APPEND*/: true,
    1025 /*O_WRONLY|O_APPEND*/: true,
    1026 /*O_RDWR|O_APPEND*/: true,
    1089 /*O_WRONLY|O_CREAT|O_APPEND*/: true,
    1090 /*O_RDWR|O_CREAT|O_APPEND*/: true,
    1153 /*O_WRONLY|O_EXCL|O_APPEND*/: true,
    1154 /*O_RDWR|O_EXCL|O_APPEND*/: true,
    1217 /*O_WRONLY|O_CREAT|O_EXCL|O_APPEND*/: true,
    1218 /*O_RDWR|O_CREAT|O_EXCL|O_APPEND*/: true,
    4096 /*O_RDONLY|O_DSYNC*/: true,
    4098 /*O_RDWR|O_DSYNC*/: true,
};
class DriveFSEmscriptenStreamOps {
    constructor(fs) {
        this.fs = fs;
    }
    open(stream) {
        const path = this.fs.realPath(stream.node);
        if (this.fs.FS.isFile(stream.node.mode)) {
            stream.file = this.fs.API.get(path);
        }
    }
    close(stream) {
        if (!this.fs.FS.isFile(stream.node.mode) || !stream.file) {
            return;
        }
        const path = this.fs.realPath(stream.node);
        const flags = stream.flags;
        let parsedFlags = typeof flags === 'string' ? parseInt(flags, 10) : flags;
        parsedFlags &= 0x1fff;
        let needsWrite = true;
        if (parsedFlags in flagNeedsWrite) {
            needsWrite = flagNeedsWrite[parsedFlags];
        }
        if (needsWrite) {
            this.fs.API.put(path, stream.file);
        }
        stream.file = undefined;
    }
    read(stream, buffer, offset, length, position) {
        if (length <= 0 ||
            stream.file === undefined ||
            position >= (stream.file.data.length || 0)) {
            return 0;
        }
        const size = Math.min(stream.file.data.length - position, length);
        buffer.set(stream.file.data.subarray(position, position + size), offset);
        return size;
    }
    write(stream, buffer, offset, length, position) {
        var _a;
        if (length <= 0 || stream.file === undefined) {
            return 0;
        }
        stream.node.timestamp = Date.now();
        if (position + length > (((_a = stream.file) === null || _a === void 0 ? void 0 : _a.data.length) || 0)) {
            const oldData = stream.file.data ? stream.file.data : new Uint8Array();
            stream.file.data = new Uint8Array(position + length);
            stream.file.data.set(oldData);
        }
        stream.file.data.set(buffer.subarray(offset, offset + length), position);
        return length;
    }
    llseek(stream, offset, whence) {
        let position = offset;
        if (whence === _emscripten__WEBPACK_IMPORTED_MODULE_0__.SEEK_CUR) {
            position += stream.position;
        }
        else if (whence === _emscripten__WEBPACK_IMPORTED_MODULE_0__.SEEK_END) {
            if (this.fs.FS.isFile(stream.node.mode)) {
                if (stream.file !== undefined) {
                    position += stream.file.data.length;
                }
                else {
                    throw new this.fs.FS.ErrnoError(this.fs.ERRNO_CODES.EPERM);
                }
            }
        }
        if (position < 0) {
            throw new this.fs.FS.ErrnoError(this.fs.ERRNO_CODES.EINVAL);
        }
        return position;
    }
}
class DriveFSEmscriptenNodeOps {
    constructor(fs) {
        this.fs = fs;
    }
    node(nodeOrStream) {
        if ((0,_emscripten__WEBPACK_IMPORTED_MODULE_0__.instanceOfStream)(nodeOrStream)) {
            return nodeOrStream.node;
        }
        return nodeOrStream;
    }
    getattr(value) {
        const node = this.node(value);
        return {
            ...this.fs.API.getattr(this.fs.realPath(node)),
            mode: node.mode,
            ino: node.id,
        };
    }
    setattr(value, attr) {
        const node = this.node(value);
        for (const [key, value] of Object.entries(attr)) {
            switch (key) {
                case 'mode':
                    node.mode = value;
                    break;
                case 'timestamp':
                    node.timestamp = value;
                    break;
                case 'size': {
                    const size = value;
                    const path = this.fs.realPath(node);
                    if (this.fs.FS.isFile(node.mode) && size >= 0) {
                        const file = this.fs.API.get(path);
                        const oldData = file.data ? file.data : new Uint8Array();
                        if (size !== oldData.length) {
                            if (size < oldData.length) {
                                file.data = file.data.slice(0, size);
                            }
                            else {
                                file.data = new Uint8Array(size);
                                file.data.set(oldData);
                            }
                            this.fs.API.put(path, file);
                        }
                    }
                    else {
                        console.warn('setattr size of', size, 'on', node, 'not yet implemented');
                    }
                    break;
                }
                default:
                    console.warn('setattr', key, 'of', value, 'on', node, 'not yet implemented');
                    break;
            }
        }
    }
    lookup(parent, name) {
        const node = this.node(parent);
        const path = this.fs.PATH.join2(this.fs.realPath(node), name);
        const result = this.fs.API.lookup(path);
        if (!result.ok) {
            throw this.fs.FS.genericErrors[this.fs.ERRNO_CODES['ENOENT']];
        }
        return this.fs.createNode(node, name, result.mode, 0);
    }
    mknod(parent, name, mode, dev) {
        const node = this.node(parent);
        const path = this.fs.PATH.join2(this.fs.realPath(node), name);
        this.fs.API.mknod(path, mode);
        return this.fs.createNode(node, name, mode, dev);
    }
    rename(value, newDir, newName) {
        const oldNode = this.node(value);
        const newDirNode = this.node(newDir);
        this.fs.API.rename(oldNode.parent
            ? this.fs.PATH.join2(this.fs.realPath(oldNode.parent), oldNode.name)
            : oldNode.name, this.fs.PATH.join2(this.fs.realPath(newDirNode), newName));
        // Updating the in-memory node
        oldNode.name = newName;
        oldNode.parent = newDirNode;
    }
    unlink(parent, name) {
        this.fs.API.rmdir(this.fs.PATH.join2(this.fs.realPath(this.node(parent)), name));
    }
    rmdir(parent, name) {
        this.fs.API.rmdir(this.fs.PATH.join2(this.fs.realPath(this.node(parent)), name));
    }
    readdir(value) {
        return this.fs.API.readdir(this.fs.realPath(this.node(value)));
    }
    symlink(parent, newName, oldPath) {
        throw new this.fs.FS.ErrnoError(this.fs.ERRNO_CODES['EPERM']);
    }
    readlink(node) {
        throw new this.fs.FS.ErrnoError(this.fs.ERRNO_CODES['EPERM']);
    }
}
/**
 * ContentsAPI base class
 */
class ContentsAPI {
    constructor(driveName, mountpoint, FS, ERRNO_CODES) {
        this._driveName = driveName;
        this._mountpoint = mountpoint;
        this.FS = FS;
        this.ERRNO_CODES = ERRNO_CODES;
    }
    lookup(path) {
        return this.request({ method: 'lookup', path: this.normalizePath(path) });
    }
    getmode(path) {
        return this.request({ method: 'getmode', path: this.normalizePath(path) });
    }
    mknod(path, mode) {
        return this.request({
            method: 'mknod',
            path: this.normalizePath(path),
            data: { mode },
        });
    }
    rename(oldPath, newPath) {
        return this.request({
            method: 'rename',
            path: this.normalizePath(oldPath),
            data: { newPath: this.normalizePath(newPath) },
        });
    }
    readdir(path) {
        const dirlist = this.request({
            method: 'readdir',
            path: this.normalizePath(path),
        });
        dirlist.push('.');
        dirlist.push('..');
        return dirlist;
    }
    rmdir(path) {
        return this.request({ method: 'rmdir', path: this.normalizePath(path) });
    }
    get(path) {
        const response = this.request({
            method: 'get',
            path: this.normalizePath(path),
        });
        if (!response) {
            throw new this.FS.ErrnoError(this.ERRNO_CODES['ENOENT']);
        }
        const serializedContent = response.content;
        const format = response.format;
        switch (format) {
            case 'json':
            case 'text':
                return {
                    data: encoder.encode(serializedContent),
                    format,
                };
            case 'base64': {
                const binString = atob(serializedContent);
                const len = binString.length;
                const data = new Uint8Array(len);
                for (let i = 0; i < len; i++) {
                    data[i] = binString.charCodeAt(i);
                }
                return {
                    data,
                    format,
                };
            }
            default:
                throw new this.FS.ErrnoError(this.ERRNO_CODES['ENOENT']);
        }
    }
    put(path, value) {
        switch (value.format) {
            case 'json':
            case 'text':
                return this.request({
                    method: 'put',
                    path: this.normalizePath(path),
                    data: {
                        format: value.format,
                        data: decoder.decode(value.data),
                    },
                });
            case 'base64': {
                let binary = '';
                for (let i = 0; i < value.data.byteLength; i++) {
                    binary += String.fromCharCode(value.data[i]);
                }
                return this.request({
                    method: 'put',
                    path: this.normalizePath(path),
                    data: {
                        format: value.format,
                        data: btoa(binary),
                    },
                });
            }
        }
    }
    getattr(path) {
        const stats = this.request({
            method: 'getattr',
            path: this.normalizePath(path),
        });
        // Turn datetimes into proper objects
        if (stats.atime) {
            stats.atime = new Date(stats.atime);
        }
        if (stats.mtime) {
            stats.mtime = new Date(stats.mtime);
        }
        if (stats.ctime) {
            stats.ctime = new Date(stats.ctime);
        }
        // ensure a non-undefined size (0 isn't great, though)
        stats.size = stats.size || 0;
        return stats;
    }
    /**
     * Normalize a Path by making it compliant for the content manager
     *
     * @param path: the path relatively to the Emscripten drive
     */
    normalizePath(path) {
        // Remove mountpoint prefix
        if (path.startsWith(this._mountpoint)) {
            path = path.slice(this._mountpoint.length);
        }
        // Add JupyterLab drive name
        if (this._driveName) {
            path = `${this._driveName}${DRIVE_SEPARATOR}${path}`;
        }
        return path;
    }
}
/**
 * An Emscripten-compatible synchronous Contents API using the service worker.
 */
class ServiceWorkerContentsAPI extends ContentsAPI {
    constructor(baseUrl, driveName, mountpoint, FS, ERRNO_CODES) {
        super(driveName, mountpoint, FS, ERRNO_CODES);
        this._baseUrl = baseUrl;
    }
    request(data) {
        const xhr = new XMLHttpRequest();
        xhr.open('POST', encodeURI(this.endpoint), false);
        try {
            xhr.send(JSON.stringify(data));
        }
        catch (e) {
            console.error(e);
        }
        if (xhr.status >= 400) {
            throw new this.FS.ErrnoError(this.ERRNO_CODES['EINVAL']);
        }
        return JSON.parse(xhr.responseText);
    }
    /**
     * Get the api/drive endpoint
     */
    get endpoint() {
        return `${this._baseUrl}api/drive`;
    }
}
class DriveFS {
    constructor(options) {
        this.FS = options.FS;
        this.PATH = options.PATH;
        this.ERRNO_CODES = options.ERRNO_CODES;
        this.API = this.createAPI(options);
        this.driveName = options.driveName;
        this.node_ops = new DriveFSEmscriptenNodeOps(this);
        this.stream_ops = new DriveFSEmscriptenStreamOps(this);
    }
    /**
     * Create the ContentsAPI.
     *
     * This is supposed to be overwritten if needed.
     */
    createAPI(options) {
        return new ServiceWorkerContentsAPI(options.baseUrl, options.driveName, options.mountpoint, options.FS, options.ERRNO_CODES);
    }
    mount(mount) {
        return this.createNode(null, mount.mountpoint, _emscripten__WEBPACK_IMPORTED_MODULE_0__.DIR_MODE | 511, 0);
    }
    createNode(parent, name, mode, dev) {
        const FS = this.FS;
        if (!FS.isDir(mode) && !FS.isFile(mode)) {
            throw new FS.ErrnoError(this.ERRNO_CODES['EINVAL']);
        }
        const node = FS.createNode(parent, name, mode, dev);
        node.node_ops = this.node_ops;
        node.stream_ops = this.stream_ops;
        return node;
    }
    getMode(path) {
        return this.API.getmode(path);
    }
    realPath(node) {
        const parts = [];
        let currentNode = node;
        parts.push(currentNode.name);
        while (currentNode.parent !== currentNode) {
            currentNode = currentNode.parent;
            parts.push(currentNode.name);
        }
        parts.reverse();
        return this.PATH.join.apply(null, parts);
    }
}


/***/ }),

/***/ "../../../node_modules/@jupyterlite/pyodide-kernel/node_modules/@jupyterlite/contents/lib/emscripten.js":
/*!**************************************************************************************************************!*\
  !*** ../../../node_modules/@jupyterlite/pyodide-kernel/node_modules/@jupyterlite/contents/lib/emscripten.js ***!
  \**************************************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "DIR_MODE": () => (/* binding */ DIR_MODE),
/* harmony export */   "FILE_MODE": () => (/* binding */ FILE_MODE),
/* harmony export */   "SEEK_CUR": () => (/* binding */ SEEK_CUR),
/* harmony export */   "SEEK_END": () => (/* binding */ SEEK_END),
/* harmony export */   "instanceOfStream": () => (/* binding */ instanceOfStream)
/* harmony export */ });
// Copyright (c) Jupyter Development Team.
// Distributed under the terms of the Modified BSD License.
/// <reference path="../../../node_modules/@types/emscripten/index.d.ts" />
const DIR_MODE = 16895; // 040777
const FILE_MODE = 33206; // 100666
const SEEK_CUR = 1;
const SEEK_END = 2;
function instanceOfStream(nodeOrStream) {
    return 'node' in nodeOrStream;
}


/***/ }),

/***/ "../../../node_modules/coincident/esm/bridge.js":
/*!******************************************************!*\
  !*** ../../../node_modules/coincident/esm/bridge.js ***!
  \******************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "SharedArrayBuffer": () => (/* binding */ SharedArrayBuffer),
/* harmony export */   "isArray": () => (/* binding */ isArray),
/* harmony export */   "notify": () => (/* binding */ notify),
/* harmony export */   "postPatched": () => (/* binding */ postPatched),
/* harmony export */   "wait": () => (/* binding */ wait),
/* harmony export */   "waitAsync": () => (/* binding */ waitAsync)
/* harmony export */ });
/* harmony import */ var _channel_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./channel.js */ "../../../node_modules/coincident/esm/channel.js");
// The goal of this file is to normalize SAB
// at least in main -> worker() use cases.
// This still cannot possibly solve the sync
// worker -> main() use case if SharedArrayBuffer
// is not available or usable.



const {isArray} = Array;

let {SharedArrayBuffer, window} = globalThis;
let {notify, wait, waitAsync} = Atomics;
let postPatched = null;

// This is needed for some version of Firefox
if (!waitAsync) {
  waitAsync = buffer => ({
    value: new Promise(onmessage => {
      // encodeURIComponent('onmessage=({data:b})=>(Atomics.wait(b,0),postMessage(0))')
      let w = new Worker('data:application/javascript,onmessage%3D(%7Bdata%3Ab%7D)%3D%3E(Atomics.wait(b%2C0)%2CpostMessage(0))');
      w.onmessage = onmessage;
      w.postMessage(buffer);
    })
  });
}

// Monkey-patch SharedArrayBuffer if needed
try {
  new SharedArrayBuffer(4);
}
catch (_) {
  SharedArrayBuffer = ArrayBuffer;

  const ids = new WeakMap;
  // patch only main -> worker():async use case
  if (window) {
    const resolvers = new Map;
    const {prototype: {postMessage}} = Worker;

    const listener = event => {
      const details = event.data?.[_channel_js__WEBPACK_IMPORTED_MODULE_0__.CHANNEL];
      if (!isArray(details)) {
        event.stopImmediatePropagation();
        const { id, sb } = details;
        resolvers.get(id)(sb);
      }
    };

    postPatched = function (data, ...rest) {
      const details = data?.[_channel_js__WEBPACK_IMPORTED_MODULE_0__.CHANNEL];
      if (isArray(details)) {
        const [id, sb] = details;
        ids.set(sb, id);
        this.addEventListener('message', listener);
      }
      return postMessage.call(this, data, ...rest);
    };

    waitAsync = sb => ({
      value: new Promise(resolve => {
        resolvers.set(ids.get(sb), resolve);
      }).then(buff => {
        resolvers.delete(ids.get(sb));
        ids.delete(sb);
        for (let i = 0; i < buff.length; i++) sb[i] = buff[i];
        return 'ok';
      })
    });
  }
  else {
    const as = (id, sb) => ({[_channel_js__WEBPACK_IMPORTED_MODULE_0__.CHANNEL]: { id, sb }});

    notify = sb => {
      postMessage(as(ids.get(sb), sb));
    };

    addEventListener('message', event => {
      const details = event.data?.[_channel_js__WEBPACK_IMPORTED_MODULE_0__.CHANNEL];
      if (isArray(details)) {
        const [id, sb] = details;
        ids.set(sb, id);
      }
    });
  }
}




/***/ }),

/***/ "../../../node_modules/coincident/esm/channel.js":
/*!*******************************************************!*\
  !*** ../../../node_modules/coincident/esm/channel.js ***!
  \*******************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "CHANNEL": () => (/* binding */ CHANNEL),
/* harmony export */   "MAIN": () => (/* binding */ MAIN),
/* harmony export */   "THREAD": () => (/* binding */ THREAD)
/* harmony export */ });
// ⚠️ AUTOMATICALLY GENERATED - DO NOT CHANGE
const CHANNEL = '64e10b34-2bf7-4616-9668-f99de5aa046e';

const MAIN = 'M' + CHANNEL;
const THREAD = 'T' + CHANNEL;


/***/ }),

/***/ "../../../node_modules/coincident/esm/index.js":
/*!*****************************************************!*\
  !*** ../../../node_modules/coincident/esm/index.js ***!
  \*****************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "default": () => (__WEBPACK_DEFAULT_EXPORT__)
/* harmony export */ });
/* harmony import */ var proxy_target_types__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! proxy-target/types */ "../../../node_modules/proxy-target/esm/types.js");
/* harmony import */ var _channel_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./channel.js */ "../../../node_modules/coincident/esm/channel.js");
/* harmony import */ var _shared_traps_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./shared/traps.js */ "../../../node_modules/coincident/esm/shared/traps.js");
/* harmony import */ var _bridge_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./bridge.js */ "../../../node_modules/coincident/esm/bridge.js");
/*! (c) Andrea Giammarchi - ISC */








// just minifier friendly for Blob Workers' cases
const {Int32Array, Map, Uint16Array} = globalThis;

// common constants / utilities for repeated operations
const {BYTES_PER_ELEMENT: I32_BYTES} = Int32Array;
const {BYTES_PER_ELEMENT: UI16_BYTES} = Uint16Array;

const waitInterrupt = (sb, delay, handler) => {
  while ((0,_bridge_js__WEBPACK_IMPORTED_MODULE_3__.wait)(sb, 0, 0, delay) === 'timed-out')
    handler();
};

// retain buffers to transfer
const buffers = new WeakSet;

// retain either main threads or workers global context
const context = new WeakMap;

const syncResult = {value: {then: fn => fn()}};

// used to generate a unique `id` per each worker `postMessage` "transaction"
let uid = 0;

/**
 * @typedef {Object} Interrupt used to sanity-check interrupts while waiting synchronously.
 * @prop {function} [handler] a callback invoked every `delay` milliseconds.
 * @prop {number} [delay=42] define `handler` invokes in terms of milliseconds.
 */

/**
 * Create once a `Proxy` able to orchestrate synchronous `postMessage` out of the box.
 * @param {globalThis | Worker} self the context in which code should run
 * @param {{parse: (serialized: string) => any, stringify: (serializable: any) => string, transform?: (value:any) => any, interrupt?: () => void | Interrupt}} [JSON] an optional `JSON` like interface to `parse` or `stringify` content with extra `transform` ability.
 * @returns {ProxyHandler<globalThis> | ProxyHandler<Worker>}
 */
const coincident = (self, {parse = JSON.parse, stringify = JSON.stringify, transform, interrupt} = JSON) => {
  // create a Proxy once for the given context (globalThis or Worker instance)
  if (!context.has(self)) {
    // ensure no SAB gets a chance to pass through this call
    const sendMessage = _bridge_js__WEBPACK_IMPORTED_MODULE_3__.postPatched || self.postMessage;
    // ensure the CHANNEL and data are posted correctly
    const post = (transfer, ...args) => sendMessage.call(self, {[_channel_js__WEBPACK_IMPORTED_MODULE_1__.CHANNEL]: args}, {transfer});

    const handler = typeof interrupt === proxy_target_types__WEBPACK_IMPORTED_MODULE_0__.FUNCTION ? interrupt : interrupt?.handler;
    const delay = interrupt?.delay || 42;
    const decoder = new TextDecoder('utf-16');

    // automatically uses sync wait (worker -> main)
    // or fallback to async wait (main -> worker)
    const waitFor = (isAsync, sb) => isAsync ?
      (0,_bridge_js__WEBPACK_IMPORTED_MODULE_3__.waitAsync)(sb, 0) :
      ((handler ? waitInterrupt(sb, delay, handler) : (0,_bridge_js__WEBPACK_IMPORTED_MODULE_3__.wait)(sb, 0)), syncResult);

    // prevent Harakiri https://github.com/WebReflection/coincident/issues/18
    let seppuku = false;

    context.set(self, new Proxy(new Map, {
      // there is very little point in checking prop in proxy for this very specific case
      // and I don't want to orchestrate a whole roundtrip neither, as stuff would fail
      // regardless if from Worker we access non existent Main callback, and vice-versa.
      // This is here mostly to guarantee that if such check is performed, at least the
      // get trap goes through and then it's up to developers guarantee they are accessing
      // stuff that actually exists elsewhere.
      [_shared_traps_js__WEBPACK_IMPORTED_MODULE_2__.HAS]: (_, action) => typeof action === 'string' && !action.startsWith('_'),

      // worker related: get any utility that should be available on the main thread
      [_shared_traps_js__WEBPACK_IMPORTED_MODULE_2__.GET]: (_, action) => action === 'then' ? null : ((...args) => {
        // transaction id
        const id = uid++;

        // first contact: just ask for how big the buffer should be
        // the value would be stored at index [1] while [0] is just control
        let sb = new Int32Array(new _bridge_js__WEBPACK_IMPORTED_MODULE_3__.SharedArrayBuffer(I32_BYTES * 2));

        // if a transfer list has been passed, drop it from args
        let transfer = [];
        if (buffers.has(args.at(-1) || transfer))
          buffers.delete(transfer = args.pop());

        // ask for invoke with arguments and wait for it
        post(transfer, id, sb, action, transform ? args.map(transform) : args);

        // helps deciding how to wait for results
        const isAsync = self !== globalThis;

        // warn users about possible deadlock still allowing them
        // to explicitly `proxy.invoke().then(...)` without blocking
        let deadlock = 0;
        if (seppuku && isAsync)
          deadlock = setTimeout(console.warn, 1000, `💀🔒 - Possible deadlock if proxy.${action}(...args) is awaited`);

        return waitFor(isAsync, sb).value.then(() => {
          clearTimeout(deadlock);

          // commit transaction using the returned / needed buffer length
          const length = sb[1];

          // filter undefined results
          if (!length) return;

          // calculate the needed ui16 bytes length to store the result string
          const bytes = UI16_BYTES * length;

          // round up to the next amount of bytes divided by 4 to allow i32 operations
          sb = new Int32Array(new _bridge_js__WEBPACK_IMPORTED_MODULE_3__.SharedArrayBuffer(bytes + (bytes % I32_BYTES)));

          // ask for results and wait for it
          post([], id, sb);
          return waitFor(isAsync, sb).value.then(() => parse(
            decoder.decode(new Uint16Array(sb.buffer).slice(0, length)))
          );
        });
      }),

      // main thread related: react to any utility a worker is asking for
      [_shared_traps_js__WEBPACK_IMPORTED_MODULE_2__.SET](actions, action, callback) {
        const type = typeof callback;
        if (type !== proxy_target_types__WEBPACK_IMPORTED_MODULE_0__.FUNCTION)
          throw new Error(`Unable to assign ${action} as ${type}`);
        // lazy event listener and logic handling, triggered once by setters actions
        if (!actions.size) {
          // maps results by `id` as they are asked for
          const results = new Map;
          // add the event listener once (first defined setter, all others work the same)
          self.addEventListener('message', async (event) => {
            // grub the very same library CHANNEL; ignore otherwise
            const details = event.data?.[_channel_js__WEBPACK_IMPORTED_MODULE_1__.CHANNEL];
            if ((0,_bridge_js__WEBPACK_IMPORTED_MODULE_3__.isArray)(details)) {
              // if early enough, avoid leaking data to other listeners
              event.stopImmediatePropagation();
              const [id, sb, ...rest] = details;
              let error;
              // action available: it must be defined/known on the main thread
              if (rest.length) {
                const [action, args] = rest;
                if (actions.has(action)) {
                  seppuku = true;
                  try {
                    // await for result either sync or async and serialize it
                    const result = await actions.get(action)(...args);
                    if (result !== void 0) {
                      const serialized = stringify(transform ? transform(result) : result);
                      // store the result for "the very next" event listener call
                      results.set(id, serialized);
                      // communicate the required SharedArrayBuffer length out of the
                      // resulting serialized string
                      sb[1] = serialized.length;
                    }
                  }
                  catch (_) {
                    error = _;
                  }
                  finally {
                    seppuku = false;
                  }
                }
                // unknown action should be notified as missing on the main thread
                else {
                  error = new Error(`Unsupported action: ${action}`);
                }
                // unlock the wait lock later on
                sb[0] = 1;
              }
              // no action means: get results out of the well known `id`
              // wait lock automatically unlocked here as no `0` value would
              // possibly ever land at index `0`
              else {
                const result = results.get(id);
                results.delete(id);
                // populate the SharedArrayBuffer with utf-16 chars code
                for (let ui16a = new Uint16Array(sb.buffer), i = 0; i < result.length; i++)
                  ui16a[i] = result.charCodeAt(i);
              }
              // release te worker waiting either the length or the result
              (0,_bridge_js__WEBPACK_IMPORTED_MODULE_3__.notify)(sb, 0);
              if (error) throw error;
            }
          });
        }
        // store this action callback allowing the setter in the process
        return !!actions.set(action, callback);
      }
    }));
  }
  return context.get(self);
};

coincident.transfer = (...args) => (buffers.add(args), args);

/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (coincident);


/***/ }),

/***/ "../../../node_modules/coincident/esm/shared/traps.js":
/*!************************************************************!*\
  !*** ../../../node_modules/coincident/esm/shared/traps.js ***!
  \************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "APPLY": () => (/* reexport safe */ proxy_target_traps__WEBPACK_IMPORTED_MODULE_0__.APPLY),
/* harmony export */   "CONSTRUCT": () => (/* reexport safe */ proxy_target_traps__WEBPACK_IMPORTED_MODULE_0__.CONSTRUCT),
/* harmony export */   "DEFINE_PROPERTY": () => (/* reexport safe */ proxy_target_traps__WEBPACK_IMPORTED_MODULE_0__.DEFINE_PROPERTY),
/* harmony export */   "DELETE": () => (/* binding */ DELETE),
/* harmony export */   "DELETE_PROPERTY": () => (/* reexport safe */ proxy_target_traps__WEBPACK_IMPORTED_MODULE_0__.DELETE_PROPERTY),
/* harmony export */   "GET": () => (/* reexport safe */ proxy_target_traps__WEBPACK_IMPORTED_MODULE_0__.GET),
/* harmony export */   "GET_OWN_PROPERTY_DESCRIPTOR": () => (/* reexport safe */ proxy_target_traps__WEBPACK_IMPORTED_MODULE_0__.GET_OWN_PROPERTY_DESCRIPTOR),
/* harmony export */   "GET_PROTOTYPE_OF": () => (/* reexport safe */ proxy_target_traps__WEBPACK_IMPORTED_MODULE_0__.GET_PROTOTYPE_OF),
/* harmony export */   "HAS": () => (/* reexport safe */ proxy_target_traps__WEBPACK_IMPORTED_MODULE_0__.HAS),
/* harmony export */   "IS_EXTENSIBLE": () => (/* reexport safe */ proxy_target_traps__WEBPACK_IMPORTED_MODULE_0__.IS_EXTENSIBLE),
/* harmony export */   "OWN_KEYS": () => (/* reexport safe */ proxy_target_traps__WEBPACK_IMPORTED_MODULE_0__.OWN_KEYS),
/* harmony export */   "PREVENT_EXTENSION": () => (/* reexport safe */ proxy_target_traps__WEBPACK_IMPORTED_MODULE_0__.PREVENT_EXTENSION),
/* harmony export */   "SET": () => (/* reexport safe */ proxy_target_traps__WEBPACK_IMPORTED_MODULE_0__.SET),
/* harmony export */   "SET_PROTOTYPE_OF": () => (/* reexport safe */ proxy_target_traps__WEBPACK_IMPORTED_MODULE_0__.SET_PROTOTYPE_OF)
/* harmony export */ });
/* harmony import */ var proxy_target_traps__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! proxy-target/traps */ "../../../node_modules/proxy-target/esm/traps.js");

const DELETE = 'delete';


/***/ }),

/***/ "../../../node_modules/proxy-target/esm/traps.js":
/*!*******************************************************!*\
  !*** ../../../node_modules/proxy-target/esm/traps.js ***!
  \*******************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "APPLY": () => (/* binding */ APPLY),
/* harmony export */   "CONSTRUCT": () => (/* binding */ CONSTRUCT),
/* harmony export */   "DEFINE_PROPERTY": () => (/* binding */ DEFINE_PROPERTY),
/* harmony export */   "DELETE_PROPERTY": () => (/* binding */ DELETE_PROPERTY),
/* harmony export */   "GET": () => (/* binding */ GET),
/* harmony export */   "GET_OWN_PROPERTY_DESCRIPTOR": () => (/* binding */ GET_OWN_PROPERTY_DESCRIPTOR),
/* harmony export */   "GET_PROTOTYPE_OF": () => (/* binding */ GET_PROTOTYPE_OF),
/* harmony export */   "HAS": () => (/* binding */ HAS),
/* harmony export */   "IS_EXTENSIBLE": () => (/* binding */ IS_EXTENSIBLE),
/* harmony export */   "OWN_KEYS": () => (/* binding */ OWN_KEYS),
/* harmony export */   "PREVENT_EXTENSION": () => (/* binding */ PREVENT_EXTENSION),
/* harmony export */   "SET": () => (/* binding */ SET),
/* harmony export */   "SET_PROTOTYPE_OF": () => (/* binding */ SET_PROTOTYPE_OF)
/* harmony export */ });
const APPLY                        = 'apply';
const CONSTRUCT                    = 'construct';
const DEFINE_PROPERTY              = 'defineProperty';
const DELETE_PROPERTY              = 'deleteProperty';
const GET                          = 'get';
const GET_OWN_PROPERTY_DESCRIPTOR  = 'getOwnPropertyDescriptor';
const GET_PROTOTYPE_OF             = 'getPrototypeOf';
const HAS                          = 'has';
const IS_EXTENSIBLE                = 'isExtensible';
const OWN_KEYS                     = 'ownKeys';
const PREVENT_EXTENSION            = 'preventExtensions';
const SET                          = 'set';
const SET_PROTOTYPE_OF             = 'setPrototypeOf';


/***/ }),

/***/ "../../../node_modules/proxy-target/esm/types.js":
/*!*******************************************************!*\
  !*** ../../../node_modules/proxy-target/esm/types.js ***!
  \*******************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "ARRAY": () => (/* binding */ ARRAY),
/* harmony export */   "BIGINT": () => (/* binding */ BIGINT),
/* harmony export */   "BOOLEAN": () => (/* binding */ BOOLEAN),
/* harmony export */   "FUNCTION": () => (/* binding */ FUNCTION),
/* harmony export */   "NULL": () => (/* binding */ NULL),
/* harmony export */   "NUMBER": () => (/* binding */ NUMBER),
/* harmony export */   "OBJECT": () => (/* binding */ OBJECT),
/* harmony export */   "STRING": () => (/* binding */ STRING),
/* harmony export */   "SYMBOL": () => (/* binding */ SYMBOL),
/* harmony export */   "UNDEFINED": () => (/* binding */ UNDEFINED)
/* harmony export */ });
const ARRAY     = 'array';
const BIGINT    = 'bigint';
const BOOLEAN   = 'boolean';
const FUNCTION  = 'function';
const NULL      = 'null';
const NUMBER    = 'number';
const OBJECT    = 'object';
const STRING    = 'string';
const SYMBOL    = 'symbol';
const UNDEFINED = 'undefined';


/***/ }),

/***/ "../../../node_modules/comlink/dist/esm/comlink.mjs":
/*!**********************************************************!*\
  !*** ../../../node_modules/comlink/dist/esm/comlink.mjs ***!
  \**********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "createEndpoint": () => (/* binding */ createEndpoint),
/* harmony export */   "expose": () => (/* binding */ expose),
/* harmony export */   "finalizer": () => (/* binding */ finalizer),
/* harmony export */   "proxy": () => (/* binding */ proxy),
/* harmony export */   "proxyMarker": () => (/* binding */ proxyMarker),
/* harmony export */   "releaseProxy": () => (/* binding */ releaseProxy),
/* harmony export */   "transfer": () => (/* binding */ transfer),
/* harmony export */   "transferHandlers": () => (/* binding */ transferHandlers),
/* harmony export */   "windowEndpoint": () => (/* binding */ windowEndpoint),
/* harmony export */   "wrap": () => (/* binding */ wrap)
/* harmony export */ });
/**
 * @license
 * Copyright 2019 Google LLC
 * SPDX-License-Identifier: Apache-2.0
 */
const proxyMarker = Symbol("Comlink.proxy");
const createEndpoint = Symbol("Comlink.endpoint");
const releaseProxy = Symbol("Comlink.releaseProxy");
const finalizer = Symbol("Comlink.finalizer");
const throwMarker = Symbol("Comlink.thrown");
const isObject = (val) => (typeof val === "object" && val !== null) || typeof val === "function";
/**
 * Internal transfer handle to handle objects marked to proxy.
 */
const proxyTransferHandler = {
    canHandle: (val) => isObject(val) && val[proxyMarker],
    serialize(obj) {
        const { port1, port2 } = new MessageChannel();
        expose(obj, port1);
        return [port2, [port2]];
    },
    deserialize(port) {
        port.start();
        return wrap(port);
    },
};
/**
 * Internal transfer handler to handle thrown exceptions.
 */
const throwTransferHandler = {
    canHandle: (value) => isObject(value) && throwMarker in value,
    serialize({ value }) {
        let serialized;
        if (value instanceof Error) {
            serialized = {
                isError: true,
                value: {
                    message: value.message,
                    name: value.name,
                    stack: value.stack,
                },
            };
        }
        else {
            serialized = { isError: false, value };
        }
        return [serialized, []];
    },
    deserialize(serialized) {
        if (serialized.isError) {
            throw Object.assign(new Error(serialized.value.message), serialized.value);
        }
        throw serialized.value;
    },
};
/**
 * Allows customizing the serialization of certain values.
 */
const transferHandlers = new Map([
    ["proxy", proxyTransferHandler],
    ["throw", throwTransferHandler],
]);
function isAllowedOrigin(allowedOrigins, origin) {
    for (const allowedOrigin of allowedOrigins) {
        if (origin === allowedOrigin || allowedOrigin === "*") {
            return true;
        }
        if (allowedOrigin instanceof RegExp && allowedOrigin.test(origin)) {
            return true;
        }
    }
    return false;
}
function expose(obj, ep = globalThis, allowedOrigins = ["*"]) {
    ep.addEventListener("message", function callback(ev) {
        if (!ev || !ev.data) {
            return;
        }
        if (!isAllowedOrigin(allowedOrigins, ev.origin)) {
            console.warn(`Invalid origin '${ev.origin}' for comlink proxy`);
            return;
        }
        const { id, type, path } = Object.assign({ path: [] }, ev.data);
        const argumentList = (ev.data.argumentList || []).map(fromWireValue);
        let returnValue;
        try {
            const parent = path.slice(0, -1).reduce((obj, prop) => obj[prop], obj);
            const rawValue = path.reduce((obj, prop) => obj[prop], obj);
            switch (type) {
                case "GET" /* MessageType.GET */:
                    {
                        returnValue = rawValue;
                    }
                    break;
                case "SET" /* MessageType.SET */:
                    {
                        parent[path.slice(-1)[0]] = fromWireValue(ev.data.value);
                        returnValue = true;
                    }
                    break;
                case "APPLY" /* MessageType.APPLY */:
                    {
                        returnValue = rawValue.apply(parent, argumentList);
                    }
                    break;
                case "CONSTRUCT" /* MessageType.CONSTRUCT */:
                    {
                        const value = new rawValue(...argumentList);
                        returnValue = proxy(value);
                    }
                    break;
                case "ENDPOINT" /* MessageType.ENDPOINT */:
                    {
                        const { port1, port2 } = new MessageChannel();
                        expose(obj, port2);
                        returnValue = transfer(port1, [port1]);
                    }
                    break;
                case "RELEASE" /* MessageType.RELEASE */:
                    {
                        returnValue = undefined;
                    }
                    break;
                default:
                    return;
            }
        }
        catch (value) {
            returnValue = { value, [throwMarker]: 0 };
        }
        Promise.resolve(returnValue)
            .catch((value) => {
            return { value, [throwMarker]: 0 };
        })
            .then((returnValue) => {
            const [wireValue, transferables] = toWireValue(returnValue);
            ep.postMessage(Object.assign(Object.assign({}, wireValue), { id }), transferables);
            if (type === "RELEASE" /* MessageType.RELEASE */) {
                // detach and deactive after sending release response above.
                ep.removeEventListener("message", callback);
                closeEndPoint(ep);
                if (finalizer in obj && typeof obj[finalizer] === "function") {
                    obj[finalizer]();
                }
            }
        })
            .catch((error) => {
            // Send Serialization Error To Caller
            const [wireValue, transferables] = toWireValue({
                value: new TypeError("Unserializable return value"),
                [throwMarker]: 0,
            });
            ep.postMessage(Object.assign(Object.assign({}, wireValue), { id }), transferables);
        });
    });
    if (ep.start) {
        ep.start();
    }
}
function isMessagePort(endpoint) {
    return endpoint.constructor.name === "MessagePort";
}
function closeEndPoint(endpoint) {
    if (isMessagePort(endpoint))
        endpoint.close();
}
function wrap(ep, target) {
    const pendingListeners = new Map();
    ep.addEventListener("message", function handleMessage(ev) {
        const { data } = ev;
        if (!data || !data.id) {
            return;
        }
        const resolver = pendingListeners.get(data.id);
        if (!resolver) {
            return;
        }
        try {
            resolver(data);
        }
        finally {
            pendingListeners.delete(data.id);
        }
    });
    return createProxy(ep, pendingListeners, [], target);
}
function throwIfProxyReleased(isReleased) {
    if (isReleased) {
        throw new Error("Proxy has been released and is not useable");
    }
}
function releaseEndpoint(ep) {
    return requestResponseMessage(ep, new Map(), {
        type: "RELEASE" /* MessageType.RELEASE */,
    }).then(() => {
        closeEndPoint(ep);
    });
}
const proxyCounter = new WeakMap();
const proxyFinalizers = "FinalizationRegistry" in globalThis &&
    new FinalizationRegistry((ep) => {
        const newCount = (proxyCounter.get(ep) || 0) - 1;
        proxyCounter.set(ep, newCount);
        if (newCount === 0) {
            releaseEndpoint(ep);
        }
    });
function registerProxy(proxy, ep) {
    const newCount = (proxyCounter.get(ep) || 0) + 1;
    proxyCounter.set(ep, newCount);
    if (proxyFinalizers) {
        proxyFinalizers.register(proxy, ep, proxy);
    }
}
function unregisterProxy(proxy) {
    if (proxyFinalizers) {
        proxyFinalizers.unregister(proxy);
    }
}
function createProxy(ep, pendingListeners, path = [], target = function () { }) {
    let isProxyReleased = false;
    const proxy = new Proxy(target, {
        get(_target, prop) {
            throwIfProxyReleased(isProxyReleased);
            if (prop === releaseProxy) {
                return () => {
                    unregisterProxy(proxy);
                    releaseEndpoint(ep);
                    pendingListeners.clear();
                    isProxyReleased = true;
                };
            }
            if (prop === "then") {
                if (path.length === 0) {
                    return { then: () => proxy };
                }
                const r = requestResponseMessage(ep, pendingListeners, {
                    type: "GET" /* MessageType.GET */,
                    path: path.map((p) => p.toString()),
                }).then(fromWireValue);
                return r.then.bind(r);
            }
            return createProxy(ep, pendingListeners, [...path, prop]);
        },
        set(_target, prop, rawValue) {
            throwIfProxyReleased(isProxyReleased);
            // FIXME: ES6 Proxy Handler `set` methods are supposed to return a
            // boolean. To show good will, we return true asynchronously ¯\_(ツ)_/¯
            const [value, transferables] = toWireValue(rawValue);
            return requestResponseMessage(ep, pendingListeners, {
                type: "SET" /* MessageType.SET */,
                path: [...path, prop].map((p) => p.toString()),
                value,
            }, transferables).then(fromWireValue);
        },
        apply(_target, _thisArg, rawArgumentList) {
            throwIfProxyReleased(isProxyReleased);
            const last = path[path.length - 1];
            if (last === createEndpoint) {
                return requestResponseMessage(ep, pendingListeners, {
                    type: "ENDPOINT" /* MessageType.ENDPOINT */,
                }).then(fromWireValue);
            }
            // We just pretend that `bind()` didn’t happen.
            if (last === "bind") {
                return createProxy(ep, pendingListeners, path.slice(0, -1));
            }
            const [argumentList, transferables] = processArguments(rawArgumentList);
            return requestResponseMessage(ep, pendingListeners, {
                type: "APPLY" /* MessageType.APPLY */,
                path: path.map((p) => p.toString()),
                argumentList,
            }, transferables).then(fromWireValue);
        },
        construct(_target, rawArgumentList) {
            throwIfProxyReleased(isProxyReleased);
            const [argumentList, transferables] = processArguments(rawArgumentList);
            return requestResponseMessage(ep, pendingListeners, {
                type: "CONSTRUCT" /* MessageType.CONSTRUCT */,
                path: path.map((p) => p.toString()),
                argumentList,
            }, transferables).then(fromWireValue);
        },
    });
    registerProxy(proxy, ep);
    return proxy;
}
function myFlat(arr) {
    return Array.prototype.concat.apply([], arr);
}
function processArguments(argumentList) {
    const processed = argumentList.map(toWireValue);
    return [processed.map((v) => v[0]), myFlat(processed.map((v) => v[1]))];
}
const transferCache = new WeakMap();
function transfer(obj, transfers) {
    transferCache.set(obj, transfers);
    return obj;
}
function proxy(obj) {
    return Object.assign(obj, { [proxyMarker]: true });
}
function windowEndpoint(w, context = globalThis, targetOrigin = "*") {
    return {
        postMessage: (msg, transferables) => w.postMessage(msg, targetOrigin, transferables),
        addEventListener: context.addEventListener.bind(context),
        removeEventListener: context.removeEventListener.bind(context),
    };
}
function toWireValue(value) {
    for (const [name, handler] of transferHandlers) {
        if (handler.canHandle(value)) {
            const [serializedValue, transferables] = handler.serialize(value);
            return [
                {
                    type: "HANDLER" /* WireValueType.HANDLER */,
                    name,
                    value: serializedValue,
                },
                transferables,
            ];
        }
    }
    return [
        {
            type: "RAW" /* WireValueType.RAW */,
            value,
        },
        transferCache.get(value) || [],
    ];
}
function fromWireValue(value) {
    switch (value.type) {
        case "HANDLER" /* WireValueType.HANDLER */:
            return transferHandlers.get(value.name).deserialize(value.value);
        case "RAW" /* WireValueType.RAW */:
            return value.value;
    }
}
function requestResponseMessage(ep, pendingListeners, msg, transfers) {
    return new Promise((resolve) => {
        const id = generateUUID();
        pendingListeners.set(id, resolve);
        if (ep.start) {
            ep.start();
        }
        ep.postMessage(Object.assign({ id }, msg), transfers);
    });
}
function generateUUID() {
    return new Array(4)
        .fill(0)
        .map(() => Math.floor(Math.random() * Number.MAX_SAFE_INTEGER).toString(16))
        .join("-");
}


//# sourceMappingURL=comlink.mjs.map


/***/ }),

/***/ "../../../node_modules/@jupyterlite/pyodide-kernel/pypi/all.json":
/*!***********************************************************************!*\
  !*** ../../../node_modules/@jupyterlite/pyodide-kernel/pypi/all.json ***!
  \***********************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

module.exports = __webpack_require__.p + "pypi/all.json";

/***/ }),

/***/ "../../../node_modules/@jupyterlite/pyodide-kernel/pypi/ipykernel-6.9.2-py3-none-any.whl":
/*!***********************************************************************************************!*\
  !*** ../../../node_modules/@jupyterlite/pyodide-kernel/pypi/ipykernel-6.9.2-py3-none-any.whl ***!
  \***********************************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

module.exports = __webpack_require__.p + "pypi/ipykernel-6.9.2-py3-none-any.whl";

/***/ }),

/***/ "../../../node_modules/@jupyterlite/pyodide-kernel/pypi/piplite-0.4.6-py3-none-any.whl":
/*!*********************************************************************************************!*\
  !*** ../../../node_modules/@jupyterlite/pyodide-kernel/pypi/piplite-0.4.6-py3-none-any.whl ***!
  \*********************************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

module.exports = __webpack_require__.p + "pypi/piplite-0.4.6-py3-none-any.whl";

/***/ }),

/***/ "../../../node_modules/@jupyterlite/pyodide-kernel/pypi/pyodide_kernel-0.4.6-py3-none-any.whl":
/*!****************************************************************************************************!*\
  !*** ../../../node_modules/@jupyterlite/pyodide-kernel/pypi/pyodide_kernel-0.4.6-py3-none-any.whl ***!
  \****************************************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

module.exports = __webpack_require__.p + "pypi/pyodide_kernel-0.4.6-py3-none-any.whl";

/***/ }),

/***/ "../../../node_modules/@jupyterlite/pyodide-kernel/pypi/widgetsnbextension-3.6.999-py3-none-any.whl":
/*!**********************************************************************************************************!*\
  !*** ../../../node_modules/@jupyterlite/pyodide-kernel/pypi/widgetsnbextension-3.6.999-py3-none-any.whl ***!
  \**********************************************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

module.exports = __webpack_require__.p + "pypi/widgetsnbextension-3.6.999-py3-none-any.whl";

/***/ }),

/***/ "../../../node_modules/@jupyterlite/pyodide-kernel/pypi/widgetsnbextension-4.0.999-py3-none-any.whl":
/*!**********************************************************************************************************!*\
  !*** ../../../node_modules/@jupyterlite/pyodide-kernel/pypi/widgetsnbextension-4.0.999-py3-none-any.whl ***!
  \**********************************************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

module.exports = __webpack_require__.p + "pypi/widgetsnbextension-4.0.999-py3-none-any.whl";

/***/ }),

/***/ "../../../node_modules/@jupyterlite/pyodide-kernel/schema/piplite.v0.schema.json":
/*!***************************************************************************************!*\
  !*** ../../../node_modules/@jupyterlite/pyodide-kernel/schema/piplite.v0.schema.json ***!
  \***************************************************************************************/
/***/ ((module) => {

module.exports = JSON.parse('{"$schema":"http://json-schema.org/draft-07/schema","$id":"https://jupyterlite-pyodide-kernel.readthedocs.org/en/latest/reference/schema/piplite-v0.html#","title":"PipLite Schema v0","description":"a schema for the warehouse-like API index","$ref":"#/definitions/top","definitions":{"top":{"type":"object","patternProperties":{".*":{"$ref":"#/definitions/a-piplite-project"}}},"a-piplite-project":{"type":"object","description":"a piplite-installable project, with one or more historical releases","properties":{"releases":{"patternProperties":{".*":{"type":"array","items":{"$ref":"#/definitions/a-piplite-distribution"}}}}}},"a-piplite-distribution":{"type":"object","properties":{"comment_text":{"type":"string"},"digests":{"type":"object","properties":{"md5":{"$ref":"#/definitions/an-md5-digest"},"sha256":{"$ref":"#/definitions/a-sha256-digest"}}},"downloads":{"type":"number"},"filename":{"type":"string"},"has_sig":{"type":"boolean"},"md5_digest":{"$ref":"#/definitions/an-md5-digest"},"packagetype":{"type":"string","enum":["bdist_wheel"]},"python_version":{"type":"string"},"requires_python":{"$ref":"#/definitions/string-or-null"},"size":{"type":"number"},"upload_time":{"type":"string","format":"date-time"},"upload_time_iso_8601":{"type":"string","format":"date-time"},"url":{"type":"string","format":"uri"},"yanked":{"type":"boolean"},"yanked_reason":{"$ref":"#/definitions/string-or-null"}}},"string-or-null":{"anyOf":[{"type":"string"},{"type":"null"}]},"an-md5-digest":{"type":"string","pattern":"[a-f0-9]{32}"},"a-sha256-digest":{"type":"string","pattern":"[a-f0-9]{64}"}}}');

/***/ })

}]);
//# sourceMappingURL=vendors-node_modules_jupyterlite_pyodide-kernel_lib_index_js.170d3836456c19b59fce.js.map